<!-- ***************************************************************** -->
<sect>The Button Widget
<!-- ***************************************************************** -->

<!-- ----------------------------------------------------------------- -->
<sect1>Normal Buttons
<p>
We've almost seen all there is to see of the button widget. It's
pretty simple. There are however two ways to create a button. You can
pass a string to the Gtk_Button ctor to create a button with a label,
or not to create a blank button. In which case it's then up to you to
pack a label or pixmap into this new button. To do this, create a new
box, and then pack your objects into this box using the usual
Gtk_Box::pack_start, and then use Gtk_Container::add to pack the box
into the button.

Here's an example of creating a button with a picture and a label in
it. I've broken up the code to create a box from the rest so you can
use it in your programs. There are further examples of using pixmaps
later in the tutorial.

<tscreen><verb>
example_include(buttons/buttons.cc)
</verb></tscreen>

The XPMLabelBox class could be used to pack xpm's and labels into
any widget that can be a container.

The Gtk_Button widget has the following signals:

<itemize>
<item> pressed
<item> released
<item> clicked
<item> enter
<item> leave
</itemize>

<!-- ----------------------------------------------------------------- -->
<sect1> Toggle Buttons
<p>
Toggle buttons are derived from normal buttons and are very similar,
except they will always be in one of two states, alternated by a
click. They may be depressed, and when you click again, they will pop
back up. Click again, and they will pop back down.

Toggle buttons are the basis for check buttons and radio buttons, as
such, many of the calls used for toggle buttons are inherited by radio
and check buttons. I will point these out when we come to them.

Gtk_ToggleButton ctors:

<tscreen><verb>
Gtk_ToggleButton();
Gtk_ToggleButton(const string &amp;label);
</verb></tscreen>

As you can imagine, these work identically to the normal button widget
ctors. The first creates a blank toggle button, and the second, a
button with a label widget already packed into it.

To retrieve the state of the toggle widget, including radio and check
buttons, we use an accessor as shown in our example below. This tests
the state of the toggle in a callback. The signal of interest emitted
to us by toggle buttons (the toggle button, check button, and radio
button widgets), is the "toggled" signal. To check the state of these
buttons, set up a signal handler to catch the toggled signal, and use
the macro to determine its state. The callback will look something
like:

<tscreen><verb>
void toggle_button_callback (Gtk_ToggleButton *button, gpointer data)
{
    if (button->get_active()) 
    {
        /* If control reaches here, the toggle button is down */
    
    } else {
    
        /* If control reaches here, the toggle button is up */
    }
}
</verb></tscreen>

<tscreen><verb>
void Gtk_ToggleButton::set_active(bool             state );
</verb></tscreen>

The above call can be used to set the state of the toggle button, and
its children the radio and check buttons. Passing in your created
button as the first argument, and a TRUE or FALSE for the second state
argument to specify whether it should be down (depressed) or up
(released). Default is up, or FALSE.

Note that when you use the Gtk_ToggleButton::set_active() method, and
the state is actually changed, it causes the "clicked" signal to be
emitted from the button.

<tscreen><verb>
void Gtk_ToggleButton::toggled ();
</verb></tscreen>

This simply toggles the button, and emits the "toggled" signal.

<!-- ----------------------------------------------------------------- -->
<sect1> Check Buttons
<p>
Check buttons inherent many properties and functions from the the
toggle buttons above, but look a little different. Rather than being
buttons with text inside them, they are small squares with the text to
the right of them. These are often used for toggling options on and
off in applications.

The two Gtk_CheckButton ctors are 

<tscreen><verb>
Gtk_CheckButton();
Gtk_CheckButton(const string &amp;label);
</verb></tscreen>

The new_with_label function creates a check button with a label beside
it.

Checking the state of the check button is identical to that of the
toggle button.

<!-- ----------------------------------------------------------------- -->
<sect1> Radio Buttons <label id="sec_Radio_Buttons">
<p>
Radio buttons are similar to check buttons except they are grouped so
that only one may be selected/depressed at a time. This is good for
places in your application where you need to select from a short list
of options.

Ctors for Gtk_RadioButton are

<tscreen><verb>
  Gtk_RadioButton(GSList *group);
  Gtk_RadioButton(GSList *group, const string &amp;label);
</verb></tscreen>

and the (preferred) sibling-based alternatives : 

<tscreen><verb>
  Gtk_RadioButton(Gtk_RadioButton &amp;button);
  Gtk_RadioButton(Gtk_RadioButton &amp;button, const string &amp;label);
</verb></tscreen>

You'll notice the extra argument to these calls. They require a group
to perform their duty properly. The group can either be specified as a
GSList (as with the GTK+ API), or, more safely, as another
Gtk_RadioButton (the new radio button will be part of the same group
as the one passed to the ctor).

The first Gtk_RadioButton of a group must be constructed using the
first kind of ctor, passing NULL as the GSList ptr value, like this :

<tscreen><verb>
Gtk_RadioButton firstButton(0);
</verb></tscreen>

You can then either extract the new button's group with 

<tscreen><verb>
GSList *group = firstButton.group();
</verb></tscreen>

and use it to create the other buttons, or (preferably) use the button 
directly :

<tscreen><verb>
Gtk_RadioButton secondButton(firstButton);
</verb></tscreen>

It is also a good idea to explicitly set which button should be the
default depressed button with:

<tscreen><verb>
void Gtk_ToggleButton::set_active(bool state );
</verb></tscreen>

This is described in the section on toggle buttons, since
Gtk_ToggleButton derives from Gtk_RadioButton, it works in exactly the
same way.

The following example creates a radio button group with three buttons.

<tscreen><verb>
example_include(radiobuttons/radiobuttons.cc)
</verb></tscreen>

<!-- TODO: check out gtk_radio_button_new_from_widget function - TRG -->

