<!-- ***************************************************************** -->
<sect> Container Widgets
<!-- ***************************************************************** -->
<sect1>The Container Overview <label id="sec_ContainerOverview">
<p>

Before getting into the various types of containers, a
short introduction is in order.  The containers in gtk--
are generally quite different from those in gtk+.  However,
they do follow the same basic definition.
Any widget which can contain another with in it is considered to a be 
a container.  However, unlike gtk+ the interface from one
to another can vary largely. 
The functionality of containers can be broken down into two
distinct groups, single item containers and multi-item containers.  

Single item containers hold up to one widget and all decend from Gtk_Bin.  
Generally if there is 
no widget contained, the contained space is left blank.  Good examples of
these containers are buttons, eventboxes, frames, and even windows.  
Adding and removing
widgets is done through the add and remove methods.  If a widget
is already held, it is replaced.  Single item containers held 
item can be accessed though the child() accessor which returns 
a pointer to the child.   (This is a proxy.)

Multi-item containers hold many items.  Good examples of these are
boxes, notebook, menus, and toolbars.  The display behavior of these 
widgets vary greatly on the type of the container.  Because they
store multiple widgets a simple add and remove is not sufficient.
Therefore, there is an accessor to a C++ list or vector construct.  This
construct looks and functions exactly like a standard STL ordered
container.  It has a defined begin() and end().  To preform 
operations, you can iterate through its children.   Insertions 
may be at the head or tail with push_front() and push_back(), or
you can insert at an arbitrary point.  The iterators also
point to a class through which manipulations of the child
properties can be done.  Thus on the surface they are just
like the STL and can be used with little knowledge of the
interface quite easily. 

One should note our implementation is quite different from gtk+.  
In gtk+, the list types
are made to appear as a list of children of all the same type,
In gtk--, each multi-container has its own unique child type.
thus hidding the auxilary information.  You can not iterate through
the children and they are often accessed only by integer indexes
instead of direct accessing the structures.   This attempt
to generalize the interface hobbles access to data and gives
inapproperate and unusable operations to many of the container types;
therefore, we will only provide basic compatiblity to their interface. 

There are some significant differences between a gtk-- contruct
and a STL one.  The biggest difference is that the input
type and output type are different.  Normally we think of a 
list as storing one type initialized by the same, ala list&lt;int&gt;.
You can of course insert any type which can be constructed to
an int on to the list and get ints back through the iterators.
In gtk--, the process of constructing a item for the list and its
internal representation are not the same.  Thus a list
has both a input type and an output type.  The output type
represents what is returned by the iterator.  For example,
in the case of a notebook, it is pages.  The input type is
called an Element.  An Element is a temporary construction which
takes a set of input and transfers it to the gtk-- container
so that the container can build the object from it.  Elements
are derived to form a factory which represents all possible
inputs to that list.  Thus a menu would allow items, submenus,
and seperators all through the same interface.  (While at the
same time it screens agains placing other widgets which should
not live in a menu.)

Elements are collected into small namespaces called Helpers. 
Each helper namespace contains all the various element types
that an interface can take.  The elements for each interface
are unique, so a ButtonElem for a toolbar won't work for
a notebook.  You should include these namespaces only for
short sections of code in which you are construction a 
widget.

This is a very powerful concept, but takes a bit of getting 
used to.  If you don't understand it yet don't worry, the
examples will make it much more clear.

<!-- ----------------------------------------------------------------- -->
<sect1> Frames <label id="sec_Frames">
<p>
Frames can be used to enclose one or a group of widgets with a box
which can optionally be labelled. The position of the label and the
style of the box can be altered to suit.

The Gtk_Frame ctor is:

<tscreen><verb>
Gtk_Frame( const string &amp;label );
</verb></tscreen>

The label is by default placed in the upper left hand corner of the
frame. A value of NULL for the <tt/label/ argument will result in no
label being displayed. The text of the label can be changed using the
next function.

<tscreen><verb>
void Gtk_Frame::set_label(const string &amp;label);
</verb></tscreen>

The position of the label can be changed using this function:

<tscreen><verb>
void Gtk_Frame::set_label_align(gfloat    xalign,
                                gfloat    yalign );
</verb></tscreen>

<tt/xalign/ and <tt/yalign/ take values between 0.0 and 1.0. <tt/xalign/
indicates the position of the label along the top horizontal of the
frame. <tt/yalign/ is not currently used. The default value of xalign
is 0.0 which places the label at the left hand end of the frame.

The next function alters the style of the box that is used to outline
the frame.

<tscreen><verb>
void Gtk_Frame::set_shadow_type(GtkShadowType  type);
</verb></tscreen>

The <tt/type/ argument can take one of the following values:
<itemize>
<item> GTK_SHADOW_NONE
<item> GTK_SHADOW_IN
<item> GTK_SHADOW_OUT
<item> GTK_SHADOW_ETCHED_IN (the default)
<item> GTK_SHADOW_ETCHED_OUT
</itemize>

The following code example illustrates the use of the Frame widget.

<tscreen><verb>
example_include(frame/frame.cc)
</verb></tscreen>

<!-- ----------------------------------------------------------------- -->   
<sect1> Aspect Frames
<p>
The aspect frame widget is like a frame widget, except that it also
enforces the aspect ratio (that is, the ratio of the width to the
height) of the child widget to have a certain value, adding extra
space if necessary. This is useful, for instance, if you want to
preview a larger image. The size of the preview should vary when the
user resizes the window, but the aspect ratio needs to always match
the original image.
  
To create a new aspect frame use:
   
<tscreen><verb>
Gtk_AspectFrame( const string label,
                 gfloat       xalign,
                 gfloat       yalign,
                 gfloat       ratio,
                 gint         obey_child);
</verb></tscreen>
   
<tt/xalign/ and <tt/yalign/ specify alignment as with Alignment
widgets. If <tt/obey_child/ is true, the aspect ratio of a child
widget will match the aspect ratio of the ideal size it requests.
Otherwise, it is given by <tt/ratio/.
   
To change the options of an existing aspect frame, you can use:
   
<tscreen><verb>
void Gtk_AspectFrame::set( gfloat          xalign,
                           gfloat          yalign,
                           gfloat          ratio,
                           gint            obey_child);
</verb></tscreen>
   
As an example, the following program uses an AspectFrame to present a
drawing area whose aspect ratio will always be 2:1, no matter how the
user resizes the top-level window.
   
<tscreen><verb>
example_include(aspectframe/aspectframe.cc)
</verb></tscreen>

<!-- ----------------------------------------------------------------- -->   
<sect1>The EventBox <label id="sec_EventBox">
<p> 
Some Gtk-- widgets don't have associated X windows, so they just draw on
their parents. Because of this, they cannot receive events and if they
are incorrectly sized, they don't clip so you can get messy
overwriting etc. If you require more from these widgets, the EventBox
is for you.

At first glance, the EventBox widget might appear to be totally
useless. It draws nothing on the screen and responds to no
events. However, it does serve a function - it provides an X window
for its child widget. This is important as many Gtk-- widgets do not
have an associated X window. Not having an X window saves memory and
improves performance, but also has some drawbacks. A widget without an
X window cannot receive events, and does not perform any clipping on
its contents. Although the name <em/EventBox/ emphasizes the
event-handling function, the widget can also be used for clipping.
(and more, see the example below).

The EventBox widget ctor is:

<tscreen><verb>
Gtk_EventBox();
</verb></tscreen>

A child widget can then be added to this EventBox:

<tscreen><verb>
    even_box.add(child_widget);
</verb></tscreen>

The following example demonstrates both uses of an EventBox - a label
is created that is clipped to a small box, and set up so that a
mouse-click on the label causes the program to exit. Resizing the
window reveals varying amounts of the label.

<tscreen><verb>
example_include(eventbox/eventbox.cc)
</verb></tscreen>

<!-- ----------------------------------------------------------------- -->   
<sect1>The Alignment widget <label id="sec_Alignment">
<p>
The alignment widget allows you to place a widget within its window at
a position and size relative to the size of the Alignment widget
itself. For example, it can be very useful for centering a widget
within the window.

There is only one method associated with the Alignment widget, other
than the ctor :

<tscreen><verb>
Gtk_Alignment( gfloat xalign,
               gfloat yalign,
               gfloat xscale,
               gfloat yscale );

void Gtk_Alignment::set( GtkAlignment *alignment,
                         gfloat        xalign,
                         gfloat        yalign,
                         gfloat        xscale,
                         gfloat        yscale );
</verb></tscreen>

The Gtk_Alignment ::set() method allows the alignment parameters of an
exisiting Alignment widget to be altered.

All four alignment parameters are floating point numbers which can
range from 0.0 to 1.0. The <tt/xalign/ and <tt/yalign/ arguments
affect the position of the widget placed within the Alignment
widget. The <tt/xscale/ and <tt/yscale/ arguments effect the amount of
space allocated to the widget.

A child widget can be added to this Alignment widget using:

<tscreen><verb>
    alignment.add(child_widget);
</verb></tscreen>

For an example of using an Alignment widget, refer to the example for
the <ref id="sec_ProgressBar" name="Progress Bar"> widget.

<!-- ----------------------------------------------------------------- -->
<sect1>Viewports <label id="sec_Viewports">
<p>
It is unlikely that you will ever need to use the Viewport widget
directly. You are much more likely to use the
<ref id="sec_ScrolledWindows" name="Scrolled Windows"> widget which
itself uses the Viewport.

A viewport widget allows you to place a larger widget within it such
that you can view a part of it at a time. It uses
<ref id="sec_Adjustment" name="Adjustments"> to define the area that
is currently in view.

Here are the Viewport ctors :

<tscreen><verb>
Gtk_Viewport( Gtk_Adjustment &amp;hadjustment,
              Gtk_Adjustment &amp;vadjustment );
Gtk_Viewport();
</verb></tscreen>

As you can see you can specify the horizontal and vertical Adjustments
that the widget is to use when you create the widget. It will create
it's own if you don't pass any arguments.

You can get and set the adjustments after the widget has been created
using the following four methods:

<tscreen><verb>
Gtk_Adjustment* Gtk_Viewport::get_hadjustment ();

Gtk_Adjustment* Gtk_Viewport::get_vadjustment ();

void Gtk_Viewport::set_hadjustment(GtkAdjustment &amp;adjustment );

void Gtk_Viewport::set_vadjustment(GtkAdjustment &amp;adjustment );
</verb></tscreen>

The only other viewport function is used to alter its appearance:

<tscreen><verb>
void Gtk_Viewport::set_shadow_type(GtkShadowType  type );
</verb></tscreen>

Possible values for the <tt/type/ parameter are:
<itemize>
<item> GTK_SHADOW_NONE,
<item> GTK_SHADOW_IN,
<item> GTK_SHADOW_OUT,
<item> GTK_SHADOW_ETCHED_IN,
<item> GTK_SHADOW_ETCHED_OUT
</itemize>

<!-- ----------------------------------------------------------------- -->
<sect1>Scrolled Windows <label id="sec_ScrolledWindows">
<p>
Scrolled windows are used to create a scrollable area inside a real
window.  You may insert any type of widget into a scrolled window, and
it will be accessible regardless of the size by using the scrollbars.

Gtk_ScrolledWindow has the following ctors :

<tscreen><verb>
Gtk_ScrolledWindow( Gtk_Adjustment *hadjustment,
                    Gtk_Adjustment *vadjustment );

Gtk_ScrolledWindow();
</verb></tscreen>

In the first ctor, the first argument is the adjustment for the
horizontal direction, and the second, the adjustment for the vertical
direction.  Most of the time, the 2nd ctor is used, where the
adjustments are created internally.

<tscreen><verb>
void Gtk_ScrolledWindow::set_policy( GtkPolicyType      hscrollbar_policy,
                                     GtkPolicyType      vscrollbar_policy );
</verb></tscreen>

This sets the policy to be used with respect to the scrollbars.
The first argument is the scrolled window you wish to change. The second
sets the policy for the horizontal scrollbar, and the third the policy for 
the vertical scrollbar.

The policy may be one of GTK_POLICY_AUTOMATIC, or GTK_POLICY_ALWAYS.
GTK_POLICY_AUTOMATIC will automatically decide whether you need
scrollbars, whereas GTK_POLICY_ALWAYS will always leave the scrollbars
there.

You can then place your object into the scrolled window using the
following method.

<tscreen><verb>
void Gtk_ScrolledWindow::add_with_viewport(const Gtk_Widget &amp;child);
</verb></tscreen>

Here is a simple example that packs 100 toggle buttons into a scrolled
window.  I've only commented on the parts that may be new to you.

<tscreen><verb>
example_include(scrolledwin/scrolledwin.cc)
</verb></tscreen>

Try playing with resizing the window. You'll notice how the scrollbars
react. You may also wish to use the Gtk_Widget::set_usize() call to set
the default size of the window or other widgets.

<!-- ----------------------------------------------------------------- -->
<sect1> Paned Window Widgets
<p>
The paned window widgets are useful when you want to divide an area
into two parts, with the relative size of the two parts controlled by
the user. A groove is drawn between the two portions with a handle
that the user can drag to change the ratio. The division can either be
horizontal (HPaned) or vertical (VPaned).
   
To create a new paned window, call one of:
   
<tscreen><verb>
Gtk_HPaned();

Gtk_VPaned();
</verb></tscreen>

After creating the paned window widget, you need to add child widgets
to its two halves. To do this, use the methods:
   
<tscreen><verb>
void Gtk_Paned::add1 (const Gtk_Widget &amp;child);

void Gtk_Paned::add2 (const Gtk_Widget &amp;child);
</verb></tscreen>
   
<tt/Gtk_Paned::add1()/ adds the child widget to the left or top half of
the paned window. <tt/Gtk_Paned::add2()/ adds the child widget to the
right or bottom half of the paned window.

A paned widget can be changed visually using the following two
methods.

<tscreen><verb>
void Gtk_Paned::set_handle_size(guint16   size);

void Gtk_Paned::set_gutter_size(guint16   size);
</verb></tscreen>

The first of these sets the size of the handle and the second sets the
size of the gutter that is between the two parts of the paned window.

As an example, we will create part of the user interface of an
imaginary email program. A window is divided into two portions
vertically, with the top portion being a list of email messages and
the bottom portion the text of the email message. Most of the program
is pretty straightforward. A couple of points to note: text can't be
added to a Text widget until it is realized. This could be done by
calling <tt/Gtk_Widget::realize()/, but as a demonstration of an
alternate technique, we connect a handler to the "realize" signal to
add the text. Also, we need to add the <tt/GTK_SHRINK/ option to some
of the items in the table containing the text window and its
scrollbars, so that when the bottom portion is made smaller, the
correct portions shrink instead of being pushed off the bottom of the
window.

<tscreen><verb>
example_include(paned/paned.cc)
</verb></tscreen>

<!-- ----------------------------------------------------------------- -->   
<sect1> Fixed Container
<p>
The Fixed container allows you to place widgets at a fixed position
within it's window, relative to it's upper left hand corner. The
position of the widgets can be changed dynamically.

There are only two methods associated with the fixed widget, aside of
the ctor :

<tscreen><verb>
Gtk_Fixed();

void Gtk_Fixed::put(const Gtk_Widget &amp;widget,
                    gint16     x,
                    gint16     y );

void Gtk_Fixed::move(const Gtk_Widget &amp;widget,
                     gint16     x,
                     gint16     y );
</verb></tscreen>

<tt/Gtk_Fixed::put/ places <tt/widget/ in the container <tt/fixed/ at
the position specified by <tt/x/ and <tt/y/.

<tt/Gtk_Fixed::move/ allows the specified widget to be moved to a new
position.

The following example illustrates how to use the Fixed Container.

<tscreen><verb>
example_include(fixed/fixed.cc)
</verb></tscreen>

<!-- ----------------------------------------------------------------- -->
<sect1> Layout Container
<p>
The Layout container is similar to the Fixed container except that it
implements an infinite (where infinity is less than 2^32) scrolling
area. Xwindows has a limitation where windows can be at most 32767
pixels wide or tall. The Layout container gets around this limitation
by doing some exotic stuff using window and bit gravities, so that you
can have smooth scrolling even when you have many child widgets in
your scrolling area.

A Layout container is created using:

<tscreen><verb>
Gtk_Layout( Gtk_Adjustment &amp;hadjustment,
            Gtk_Adjustment &amp;vadjustment );
!!!!!!!!!!
</verb></tscreen>

As you can see, you can optionally specify the Adjustment objects that
the Layout widget will use for it's scrolling.

You can add and move widgets in the Layout container using the
following two methods:

<tscreen><verb>
void Gtk_Layout::put(const Gtk_Widget &amp;widget,
                     gint       x,
                     gint       y);

void Gtk_Layout::move(const Gtk_Widget &amp;widget,
                      gint       x,
                      gint       y);
</verb></tscreen>

The size of the Layout container can be set using the next function:

<tscreen><verb>
void Gtk_Layout::set_size(guint      width,
                          guint      height);
</verb></tscreen>

Layout containers are one of the very few widgets in the Gtk-- widget
set that actively repaint themselves on screen as they are changed
using the above functions (the vast majority of widgets queue
requests which are then processed when control returns to the
<tt/gtk_main()/ function).

When you want to make a large number of changes to a Layout container,
you can use the following two functions to disable and re-enable this
repainting functionality:

<tscreen><verb>
void Gtk_Layout::freeze();

void Gtk_Layout::thaw();
</verb></tscreen>

The final four functions for use with Layout widgets are for
manipulating the horizontal and vertical adjustment widgets:

<tscreen><verb>
Gtk_Adjustment* Gtk_Layout::get_hadjustment();

Gtk_Adjustment* Gtk_Layout::get_vadjustment();

void Gtk_Layout::set_hadjustment(Gtk_Adjustment &amp;adjustment);

void Gtk_Layout::set_vadjustment(Gtk_Adjustment &amp;adjustment);
</verb></tscreen>

<!-- ----------------------------------------------------------------- -->   
<sect1>Button Boxes
<p>
Button Boxes are a convenient way to quickly layout a group of
buttons. They come in both horizontal and vertical flavours. You
create a new Button Box with one of the following ctor, which create
a horizontal or vertical box, respectively:

<tscreen><verb>
Gtk_HButtonBox();

Gtk_VButtonBox();
</verb></tscreen>

The only attributes pertaining to button boxes effect how the buttons
are layed out. You can change the spacing between the buttons with:

<tscreen><verb>
void Gtk_HButtonBox::set_spacing_default( gint spacing );

void Gtk_VButtonBox::set_spacing_default( gint spacing );
</verb></tscreen>

Similarly, the current spacing values can be queried using:

<tscreen><verb>
gint Gtk_HButtonBox::get_spacing_default();

gint Gtk_VButton_box::get_spacing_default();
</verb></tscreen>

The second attribute that we can access effects the layour of the
buttons within the box. It is set using one of:

<tscreen><verb>
void Gtk_HButtonBox::set_layout_default( GtkButtonBoxStyle layout );

void Gtk_VButton_box::set_layout_default( GtkButtonBoxStyle layout );
</verb></tscreen>

The <tt/layout/ argument can take one of the following values:

<itemize>
<item> GTK_BUTTONBOX_DEFAULT_STYLE
<item> GTK_BUTTONBOX_SPREAD
<item> GTK_BUTTONBOX_EDGE
<item> GTK_BUTTONBOX_START
<item> GTK_BUTTONBOX_END
</itemize>

The current layout setting can be retrieved using:

<tscreen><verb>
GtkButtonBoxStyle Gtk_HButtonBox::get_layout_default();

GtkButtonBoxStyle Gtk_VButtonBox::get_layout_default();
</verb></tscreen>

Buttons are added to a Button Box using the usual method:

<tscreen><verb>
    button_box.add( child_widget );
</verb></tscreen>

Here's an example that illustrates all the different layout settings
for Button Boxes.

<tscreen><verb>
example_include(buttonbox/buttonbox.cc)
</verb></tscreen>

<!-- ----------------------------------------------------------------- -->   
<sect1>Toolbar
<p>
Toolbars are usually used to group some number of widgets in order to
simplify customization of their look and layout. Typically a toolbar
consists of buttons with icons, labels and tooltips, but any other
widget can also be put inside a toolbar. Finally, items can be
arranged horizontally or vertically and buttons can be displayed with
icons, labels or both.

Creating a toolbar is (as one may already suspect) done with the
following ctor:

<tscreen><verb>
Gtk_Toolbar( GtkOrientation orientation,
             GtkToolbarStyle  style );
</verb></tscreen>

where orientation may be one of:

<tscreen><verb>
  GTK_ORIENTATION_HORIZONTAL    
  GTK_ORIENTATION_VERTICAL
</verb></tscreen>

and style one of:

<tscreen><verb>
  GTK_TOOLBAR_TEXT
  GTK_TOOLBAR_ICONS
  GTK_TOOLBAR_BOTH
</verb></tscreen>

divert(-1)
!!!!!!!!!! The Gtk_Toolbar interface is completely different 

The style applies to all the buttons created with the `item' functions
(not to buttons inserted into toolbar as separate widgets).

After creating a toolbar one can append, prepend and insert items
(that means simple buttons) into the toolbar. To describe an item we
need a label text, a tooltip text, a private tooltip text, an icon for
the button and a callback function for it. For example, to append or
prepend an item you may use the following functions:

<tscreen><verb>
GtkWidget *gtk_toolbar_append_item( GtkToolbar    *toolbar,
                                    const char    *text,
                                    const char    *tooltip_text,
                                    const char    *tooltip_private_text,
                                    GtkWidget     *icon,
                                    GtkSignalFunc  callback,
                                    gpointer       user_data );

GtkWidget *gtk_toolbar_prepend_item( GtkToolbar    *toolbar,
                                     const char    *text,
                                     const char    *tooltip_text,
                                     const char    *tooltip_private_text,
                                     GtkWidget     *icon,
                                     GtkSignalFunc  callback,
                                     gpointer       user_data );
</verb></tscreen>

If you want to use gtk_toolbar_insert_item, the only additional
parameter which must be specified is the position in which the item
should be inserted, thus:

<tscreen><verb>
GtkWidget *gtk_toolbar_insert_item( GtkToolbar    *toolbar,
                                    const char    *text,
                                    const char    *tooltip_text,
                                    const char    *tooltip_private_text,
                                    GtkWidget     *icon,
                                    GtkSignalFunc  callback,
                                    gpointer       user_data,
                                    gint           position );
</verb></tscreen>

To simplify adding spaces between toolbar items, you may use the
following functions:

<tscreen><verb>
void gtk_toolbar_append_space( GtkToolbar *toolbar );

void gtk_toolbar_prepend_space( GtkToolbar *toolbar );

void gtk_toolbar_insert_space( GtkToolbar *toolbar,
                               gint        position );
 
</verb></tscreen>

While the size of the added space can be set globally for a
whole toolbar with the method:

<tscreen><verb>
void gtk_toolbar_set_space_size( GtkToolbar *toolbar,
                                 gint        space_size) ;
</verb></tscreen>

If it's required, the orientation of a toolbar and its style can be
changed `on the fly' using the following method:

<tscreen><verb>
void Gtk_Toolbar::set_orientation(GtkOrientation  orientation );

void Gtk_Toolbar::set_style(GtkToolbarStyle  style );

void Gtk_Toolbar::set_tooltips(gint        enable );
</verb></tscreen>

Where <tt/orientation/ is one of GTK_ORIENTATION_HORIZONTAL or
GTK_ORIENTATION_VERTICAL. The <tt/style/ is used to set appearance of
the toolbar items by using one of GTK_TOOLBAR_ICONS, GTK_TOOLBAR_TEXT
or GTK_TOOLBAR_BOTH.

To show some other things that can be done with a toolbar, let's take
the following program (we'll interrupt the listing with some
additional explanations):

<tscreen><verb>
#include <gtk--.h>
!!!!!!!!!!
#include "gtk.xpm"

/* This method is connected to the Close button or
 * closing the window from the WM */
int MyClass::delete_event_handler(GdkEventAny *event, gpointer data)
{
  gtk_main_quit ();
}
</verb></tscreen>

The above beginning seems for sure familiar to you if it's not your first
Gtk-- program. There is one additional thing though, we include a nice XPM
picture to serve as an icon for all of the buttons.

<tscreen><verb>
GtkWidget* close_button; /* This button will emit signal to close
                          * application */
GtkWidget* tooltips_button; /* to enable/disable tooltips */
GtkWidget* text_button,
         * icon_button,
         * both_button; /* radio buttons for toolbar style */
GtkWidget* entry; /* a text entry to show packing any widget into
                   * toolbar */
</verb></tscreen>

In fact not all of the above widgets are needed here, but to make things
clearer I put them all together.

<tscreen><verb>
/* that's easy... when one of the buttons is toggled, we just
 * check which one is active and set the style of the toolbar
 * accordingly
 * ATTENTION: our toolbar is passed as data to callback ! */
void radio_event (GtkWidget *widget, gpointer data)
{
  if (GTK_TOGGLE_BUTTON (text_button)->active) 
    gtk_toolbar_set_style(GTK_TOOLBAR ( data ), GTK_TOOLBAR_TEXT);
  else if (GTK_TOGGLE_BUTTON (icon_button)->active)
    gtk_toolbar_set_style(GTK_TOOLBAR ( data ), GTK_TOOLBAR_ICONS);
  else if (GTK_TOGGLE_BUTTON (both_button)->active)
    gtk_toolbar_set_style(GTK_TOOLBAR ( data ), GTK_TOOLBAR_BOTH);
}

/* even easier, just check given toggle button and enable/disable 
 * tooltips */
void toggle_event (GtkWidget *widget, gpointer data)
{
  gtk_toolbar_set_tooltips (GTK_TOOLBAR ( data ),
                            GTK_TOGGLE_BUTTON (widget)->active );
}
</verb></tscreen>

The above are just two callback functions that will be called when
one of the buttons on a toolbar is pressed. You should already be
familiar with things like this if you've already used toggle buttons (and
radio buttons).

<tscreen><verb>
int main (int argc, char *argv[])
{
  /* Here is our main window (a dialog) and a handle for the handlebox */
  GtkWidget* dialog;
  GtkWidget* handlebox;

  /* Ok, we need a toolbar, an icon with a mask (one for all of 
     the buttons) and an icon widget to put this icon in (but 
     we'll create a separate widget for each button) */
  GtkWidget * toolbar;
  GdkPixmap * icon;
  GdkBitmap * mask;
  GtkWidget * iconw;

  /* this is called in all Gtk-- application. */
  Gtk_Main kit (argc, argv);
  
  /* create a new window with a given title, and nice size */
  dialog = gtk_dialog_new ();
  gtk_window_set_title ( GTK_WINDOW ( dialog ) , "GTKToolbar Tutorial");
  gtk_widget_set_usize( GTK_WIDGET ( dialog ) , 600 , 300 );
  GTK_WINDOW ( dialog ) ->allow_shrink = TRUE;

  /* typically we quit if someone tries to close us */
  gtk_signal_connect ( GTK_OBJECT ( dialog ), "delete_event",
                       GTK_SIGNAL_FUNC ( delete_event ), NULL);

  /* we need to realize the window because we use pixmaps for 
   * items on the toolbar in the context of it */
  gtk_widget_realize ( dialog );

  /* to make it nice we'll put the toolbar into the handle box, 
   * so that it can be detached from the main window */
  handlebox = gtk_handle_box_new ();
  gtk_box_pack_start ( GTK_BOX ( GTK_DIALOG(dialog)->vbox ),
                       handlebox, FALSE, FALSE, 5 );
</verb></tscreen>

The above should be similar to any other Gtk-- application. Just
initialization of Gtk--, creating the window etc.. There is only one
thing that probably needs some explanation: a handle box. A handle box
is just another box that can be used to pack widgets in to. The
difference between it and typical boxes is that it can be detached
from a parent window (or, in fact, the handle box remains in the
parent, but it is reduced to a very small rectangle, while all of its
contents are reparented to a new freely floating window). It is
usually nice to have a detachable toolbar, so these two widgets occur
together quite often.

<tscreen><verb>
  /* toolbar will be horizontal, with both icons and text, and
   * with 5pxl spaces between items and finally, 
   * we'll also put it into our handlebox */
  toolbar = gtk_toolbar_new ( GTK_ORIENTATION_HORIZONTAL,
                              GTK_TOOLBAR_BOTH );
  gtk_container_set_border_width ( GTK_CONTAINER ( toolbar ) , 5 );
  gtk_toolbar_set_space_size ( GTK_TOOLBAR ( toolbar ), 5 );
  gtk_container_add ( GTK_CONTAINER ( handlebox ) , toolbar );

  /* now we create icon with mask: we'll reuse it to create
   * icon widgets for toolbar items */
  icon = gdk_pixmap_create_from_xpm_d ( dialog->window, &amp;mask,
      &amp;dialog->style->white, gtk_xpm );
</verb></tscreen>

Well, what we do above is just a straight-forward initialization of
the toolbar widget and creation of a GDK pixmap with its mask. If you
want to know something more about using pixmaps, refer to GDK
documentation or to the <ref id="sec_Pixmaps" name="Pixmaps"> section
earlier in this tutorial.

<tscreen><verb>
  /* our first item is <close> button */
  iconw = gtk_pixmap_new ( icon, mask ); /* icon widget */
  close_button = 
    gtk_toolbar_append_item ( GTK_TOOLBAR (toolbar), /* our toolbar */
                              "Close",               /* button label */
                              "Closes this app",     /* this button's tooltip */
                              "Private",             /* tooltip private info */
                              iconw,                 /* icon widget */
                              GTK_SIGNAL_FUNC (delete_event), /* a signal */
                               NULL );
  gtk_toolbar_append_space ( GTK_TOOLBAR ( toolbar ) ); /* space after item */
</verb></tscreen>

In the above code you see the simplest case: adding a button to
toolbar.  Just before appending a new item, we have to construct a
pixmap widget to serve as an icon for this item; this step will have
to be repeated for each new item. Just after the item we also add a
space, so the following items will not touch each other. As you see
gtk_toolbar_append_item returns a pointer to our newly created button
widget, so that we can work with it in the normal way.

<tscreen><verb>
  /* now, let's make our radio buttons group... */
  iconw = gtk_pixmap_new ( icon, mask );
  icon_button = gtk_toolbar_append_element(
                    GTK_TOOLBAR(toolbar),
                    GTK_TOOLBAR_CHILD_RADIOBUTTON, /* a type of element */
                    NULL,                          /* pointer to widget */
                    "Icon",                        /* label */
                    "Only icons in toolbar",       /* tooltip */
                    "Private",                     /* tooltip private string */
                    iconw,                         /* icon */
                    GTK_SIGNAL_FUNC (radio_event), /* signal */
                    toolbar);                      /* data for signal */
  gtk_toolbar_append_space ( GTK_TOOLBAR ( toolbar ) );
</verb></tscreen>

Here we begin creating a radio buttons group. To do this we use
gtk_toolbar_append_element.  In fact, using this function one can also
add simple items or even spaces (type = GTK_TOOLBAR_CHILD_SPACE or
GTK_TOOLBAR_CHILD_BUTTON). In the above case we start creating a radio
group. In creating other radio buttons for this group a pointer to the
previous button in the group is required, so that a list of buttons
can be easily constructed (see the section on <ref
id="sec_Radio_Buttons" name="Radio Buttons"> earlier in this
tutorial).

<tscreen><verb>
  /* following radio buttons refer to previous ones */
  iconw = gtk_pixmap_new ( icon, mask );
  text_button = 
    gtk_toolbar_append_element(GTK_TOOLBAR(toolbar),
                               GTK_TOOLBAR_CHILD_RADIOBUTTON,
                               icon_button,
                               "Text",
                               "Only texts in toolbar",
                               "Private",
                               iconw,
                               GTK_SIGNAL_FUNC (radio_event),
                               toolbar);
  gtk_toolbar_append_space ( GTK_TOOLBAR ( toolbar ) );
                                          
  iconw = gtk_pixmap_new ( icon, mask );
  both_button = 
    gtk_toolbar_append_element(GTK_TOOLBAR(toolbar),
                               GTK_TOOLBAR_CHILD_RADIOBUTTON,
                               text_button,
                               "Both",
                               "Icons and text in toolbar",
                               "Private",
                               iconw,
                               GTK_SIGNAL_FUNC (radio_event),
                               toolbar);
  gtk_toolbar_append_space ( GTK_TOOLBAR ( toolbar ) );
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(both_button),TRUE);
</verb></tscreen>

In the end we have set the state of one of the buttons manually (otherwise
they all stay in active state, preventing us from switching between them).

<tscreen><verb>
  /* here we have just a simple toggle button */
  iconw = gtk_pixmap_new ( icon, mask );
  tooltips_button = 
    gtk_toolbar_append_element(GTK_TOOLBAR(toolbar),
                               GTK_TOOLBAR_CHILD_TOGGLEBUTTON,
                               NULL,
                               "Tooltips",
                               "Toolbar with or without tips",
                               "Private",
                               iconw,
                               GTK_SIGNAL_FUNC (toggle_event),
                               toolbar);
  gtk_toolbar_append_space ( GTK_TOOLBAR ( toolbar ) );
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tooltips_button),TRUE);
</verb></tscreen>

A toggle button can be created in the obvious way (if one knows how to create
radio buttons already).

<tscreen><verb>
  /* to pack a widget into toolbar, we only have to 
   * create it and append it with an appropriate tooltip */
  entry = gtk_entry_new ();
  gtk_toolbar_append_widget( GTK_TOOLBAR (toolbar), 
                             entry, 
                             "This is just an entry", 
                             "Private" );

  /* well, it isn't created within thetoolbar, so we must still show it */
  gtk_widget_show ( entry );
</verb></tscreen>

As you see, adding any kind of widget to a toolbar is simple. The
one thing you have to remember is that this widget must be shown manually
(contrary to other items which will be shown together with the toolbar).

<tscreen><verb>
  /* that's it ! let's show everything. */
  gtk_widget_show ( toolbar );
  gtk_widget_show (handlebox);
  gtk_widget_show ( dialog );

  /* rest in gtk_main and wait for the fun to begin! */
  gtk_main ();
  
  return 0;
}
</verb></tscreen>

So, here we are at the end of toolbar tutorial. Of course, to appreciate
it in full you need also this nice XPM icon, so here it is:

<tscreen><verb>
/* XPM */
static char * gtk_xpm[] = {
"32 39 5 1",
".      c none",
"+      c black",
"@      c #3070E0",
"#      c #F05050",
"$      c #35E035",
"................+...............",
"..............+++++.............",
"............+++++@@++...........",
"..........+++++@@@@@@++.........",
"........++++@@@@@@@@@@++........",
"......++++@@++++++++@@@++.......",
".....+++@@@+++++++++++@@@++.....",
"...+++@@@@+++@@@@@@++++@@@@+....",
"..+++@@@@+++@@@@@@@@+++@@@@@++..",
".++@@@@@@+++@@@@@@@@@@@@@@@@@@++",
".+#+@@@@@@++@@@@+++@@@@@@@@@@@@+",
".+##++@@@@+++@@@+++++@@@@@@@@$@.",
".+###++@@@@+++@@@+++@@@@@++$$$@.",
".+####+++@@@+++++++@@@@@+@$$$$@.",
".+#####+++@@@@+++@@@@++@$$$$$$+.",
".+######++++@@@@@@@++@$$$$$$$$+.",
".+#######+##+@@@@+++$$$$$$@@$$+.",
".+###+++##+##+@@++@$$$$$$++$$$+.",
".+###++++##+##+@@$$$$$$$@+@$$@+.",
".+###++++++#+++@$$@+@$$@++$$$@+.",
".+####+++++++#++$$@+@$$++$$$$+..",
".++####++++++#++$$@+@$++@$$$$+..",
".+#####+++++##++$$++@+++$$$$$+..",
".++####+++##+#++$$+++++@$$$$$+..",
".++####+++####++$$++++++@$$$@+..",
".+#####++#####++$$+++@++++@$@+..",
".+#####++#####++$$++@$$@+++$@@..",
".++####++#####++$$++$$$$$+@$@++.",
".++####++#####++$$++$$$$$$$$+++.",
".+++####+#####++$$++$$$$$$$@+++.",
"..+++#########+@$$+@$$$$$$+++...",
"...+++########+@$$$$$$$$@+++....",
".....+++######+@$$$$$$$+++......",
"......+++#####+@$$$$$@++........",
".......+++####+@$$$$+++.........",
".........++###+$$$@++...........",
"..........++##+$@+++............",
"...........+++++++..............",
".............++++..............."};
</verb></tscreen>

divert(0)

<!-- ----------------------------------------------------------------- -->
<sect1> Notebooks
<p>
The NoteBook Widget is a collection of 'pages' that overlap each
other, each page contains different information. This widget has
become more common lately in GUI programming, and it is a good way to
show blocks of similar information that warrant separation in their
display.

Here's the notebook widget ctor :

<tscreen><verb>
Gtk_Notebook();
</verb></tscreen>

To begin lets look at the container list held in a notebook.
A child of a notebook is called a page.  The collection of
the pages is modeled as a list called PageList which contains
pointers to each page.  You can access it by calling

<tscreen><verb>
PageList&amp; Gtk_Notebook::pages();
</verb></tscreen>

To insert a new page, you use elements which contain a 
widget with an optional tab label and menu label.  WidgetElem is
used for just a plain widget usually a container like
a box.  TabElem allows specification with a labeled
tab.  The label on the tab can be created manually or
you can have it created automatically by providing 
a string.  The MenuElem provides the same thing for a
page with a default tab label but with a menu label.  
Use the base class Element if you wish to specify all
of the options.

Insertions are made to the page list through the standard
STL methods.

<tscreen><verb>
iterator PageList::insert(iterator location,const Element&amp; elem);
void PageList::push_front(const Element&amp; elem);
void PageList::push_back(const Element&amp; elem);
</verb></tscreen>

Removing an item can be done by the widget, page or iterator.
 
<tscreen><verb>
void PageList::erase(iterator location);
void PageList::remove(Gtk_Widget&amp;);
void PageList::remove(Page *);
</verb></tscreen>

Once a notebook has been created, there are a number of methods
that operate on the notebook widget. Let's look at them individually.

The first one we will look at is how to position the page indicators.
These page indicators or 'tabs' as they are referred to, can be
positioned in four ways: top, bottom, left, or right.

<tscreen><verb>
void Gtk_Notebook::set_tab_pos( GtkPositionType  pos );
</verb></tscreen>

GtkPostionType will be one of the following, which are pretty self
explanatory:
<itemize>
<item> GTK_POS_LEFT
<item> GTK_POS_RIGHT
<item> GTK_POS_TOP
<item> GTK_POS_BOTTOM
</itemize>

GTK_POS_TOP is the default.

To find out what the current page is in a notebook use the method:

<tscreen><verb>
Page* Gtk_Notebook::get_current() const;
gint Gtk_Notebook::get_current_page_num() const
</verb></tscreen>

These next two methods are simple calls to move the notebook page
forward or backward. Simply provide the respective method call with
the notebook widget you wish to operate on. Note: when the NoteBook is
currently on the last page, and Gtk_Notebook::next_page is called, the
notebook will wrap back to the first page. Likewise, if the NoteBook
is on the first page, and Gtk_Notebook::prev_page is called, the
notebook will wrap to the last page.

<tscreen><verb>
void Gtk_Notebook::next_page();

void Gtk_Notebook::prev_page();
</verb></tscreen>

This next method sets the 'active' page. If you wish the notebook to
be opened to page 5 for example, you would use this method.  Without
using this method, the notebook defaults to the first page.

<tscreen><verb>
void Gtk_Notebook::set_page(gint         page_num );
</verb></tscreen>

The next two method add or remove the notebook page tabs and the
notebook border respectively.

<tscreen><verb>
void Gtk_Notebook::set_show_tabs(bool     show_tabs);

void Gtk_Notebook::set_show_border(bool     show_border);
</verb></tscreen>

The next method is useful when the you have a large number of pages,
and the tabs don't fit on the page. It allows the tabs to be scrolled
through using two arrow buttons.

<tscreen><verb>
void Gtk_Notebook::set_scrollable(bool     scrollable);
</verb></tscreen>

<tt/show_tabs/, <tt/show_border/ and <tt/scrollable/ can be either
TRUE or FALSE.

Now lets look at an example, it is expanded from the testgtk.c code
that comes with the Gtk-- distribution. This small program creates a
window with a notebook and six buttons. The notebook contains 11
pages, added in three different ways, appended, inserted, and
prepended. The buttons allow you rotate the tab positions, add/remove
the tabs and border, remove a page, change pages in both a forward and
backward manner, and exit the program.

<tscreen><verb>
example_include(notebook/notebook.cc)
</verb></tscreen>

Hopefully this helps you on your way with creating notebooks for your
Gtk-- applications.
