<!-- ***************************************************************** -->
<sect>Getting Started
<!-- ***************************************************************** -->

<p>
The first thing to do of course, is download the Gtk-- source.  
If you don't already have it, you will also need to build and
install the companion library, libsigc++.
You can always get the latest version from
http://lazy.ton.tut.fi/gtk--/download.html. Gtk-- uses GNU
autoconf for configuration.  Once untar'd, type ./configure --help to
see a list of options.

Th Gtk-- source distribution also contains the complete source to some
of the examples used in this tutorial, along with Makefiles to aid
compilation.

To begin our introduction to Gtk--, we'll start with the simplest
program possible. This program will create a 200x200 pixel window and
has no way of exiting except to be killed using the shell.

<tscreen><verb>
example_include(base/base.cc)
</verb></tscreen>

You can compile the above program with gcc using:
<tscreen><verb>
g++ base.cc -o base `gtkmm-config --cflags --libs`
</verb></tscreen>

The meaning of the unusual compilation options is explained below.

All programs will of course include the various gtk-- headers.
gtk--.h includes the entire gtk-- kit.  To show quickly which
widgets are used in the exmple,
we will only include the headers required for each demo.

The next line:

<tscreen><verb>
Gtk_Main m(argc, argv);
</verb></tscreen>

creates a Gtk_Main object which will be needed in all Gtk--
applications. This function initializes the library for use, sets up
default signal handlers, and checks the arguments passed to your
application on the command line, looking for one of the following:

<itemize>
<item> <tt/--gtk-module/
<item> <tt/--g-fatal-warnings/
<item> <tt/--gtk-debug/
<item> <tt/--gtk-no-debug/
<item> <tt/--gdk-debug/
<item> <tt/--gdk-no-debug/
<item> <tt/--display/
<item> <tt/--sync/
<item> <tt/--no-xshm/
<item> <tt/--name/
<item> <tt/--class/
</itemize>

It removes these from the argument list, leaving anything it does not
recognize for your application to parse or ignore. This creates a set
of standard arguments accepted by all Gtk-- applications.

The next two lines of code create and display a window.

<tscreen><verb>
  Gtk_Window window (GTK_WINDOW_TOPLEVEL);
  window.show();
</verb></tscreen>

The GTK_WINDOW_TOPLEVEL argument specifies that we want the window to
undergo window manager decoration and placement. Rather than create a
window of 0x0 size, a window without children is set to 200x200 by
default so you can still manipulate it.

The show() method lets Gtk-- know that we are done setting
the attributes of this widget, and that it can display it.

The last line enters the Gtk-- main processing loop.

<tscreen><verb>
  m.run();
</verb></tscreen>

This is another call you will see in every Gtk-- application (the
run() method of the Gtk_Main class).  When control reaches this point,
Gtk-- will sleep waiting for X events (such as button or key presses),
timeouts, or file IO notifications to occur. In our simple example
however, events are ignored.

<!-- ----------------------------------------------------------------- -->
<sect1>Hello World in Gtk--
<p>
Now for a program with a widget (a button).  It's the classic
hello world a la Gtk--.

<tscreen><verb>
example_include(helloworld/helloworld.cc)
</verb></tscreen>

<!-- ----------------------------------------------------------------- -->
<sect1>Compiling Hello World
<p>
To compile use:

<tscreen><verb>
g++ -Wall -g helloworld.cc -o helloworld `gtkmm-config --cflags --libs`
</verb></tscreen>

This uses the program <tt>gtkmm-config</>, which comes with Gtk--. This
program 'knows' what compiler switches are needed to compile programs
that use Gtk--. <tt>gtkmm-config --cflags</> will output a list of include
directories for the compiler to look in, and <tt>gtkmm-config --libs</>
will output the list of libraries for the compiler to link with and
the directories to find them in. In the above example they have
been combined into a single instance, such as
`gtk-config --cflags --libs`.

Note that the type of single quote used in the compile command above
is significant. You must use backquotes here, but you know that if
you're shell-literate.

The libraries that are usually linked in are:
<itemize>
<item>The Gtk-- library (-lgtkmm)
<item>The Gdk-- library (-lgdkmm)
<item>The GTK library (-lgtk), the widget library, based on top of GDK.
<item>The GDK library (-lgdk), the Xlib wrapper.
<item>The gmodule library (-lgmodule), which is used to load run time
extensions.
<item>The Xlib library (-lX11) which is used by GDK.
<item>The Xext library (-lXext). This contains code for shared memory
pixmaps and other X extensions.
<item>The math library (-lm). This is used by GTK for various purposes.
</itemize>

<!-- ----------------------------------------------------------------- -->
<sect1>Theory of Signals and Callbacks
<p>
Before we look in detail at <em>helloworld</em>, we'll discuss signals
and callbacks. Gtk-- is an event driven toolkit, which means it will
sleep until an event occurs and control is passed to the appropriate
function.

This passing of control is done using the idea of "signals". When an
event occurs, such as the press of a mouse button, the appropriate
signal will be "emitted" by the widget that was pressed.  This is how
Gtk-- does most of its useful work. There are a set of signals that all
widgets inherit, such as "destroy", and there are signals that are
widget specific, such as "toggled" on a toggle button.

Also, since Gtk-- uses Karl Nelson's libsigc++, you can also create
your own signals.

To make a button perform an action, we set up a signal handler to
catch these signals and call the appropriate function. This is done by
using a method in the signal object such as:

<tscreen><verb>
signal.connect( Slot );
</verb></tscreen>

Slot is a abstract callback type built by libsigc++.  To build
one you use the slot() factory function.  Slots can be built
from functions or methods as shown.

<tscreen><verb>
void func();
struct Foo : public SigC::Object
  {
    void mymethod();
    void one_arg(int);
  } f;

Gtk_Button b;
b.clicked.connect( slot(&amp;func) );
b.clicked.connect( slot(f,&amp;Foo::mymethod) );
b.clicked.connect( bind( slot(f,&amp;Foo::one_arg), 1) );
</verb></tscreen>

The last of these connections is special as the slot was
altered using an adaptor called bind.  bind() takes the
slot created and binds arguments starting from the last
to a specific value.  This can be used to append callback
data to a slot.

To find out what type of slot you can connect to a 
signal you should look at the signal declaration.  A signal
declaration looks something like this.

<tscreen><verb>
EmitProxySignal1&lt;gint,GtkDirectionType,
                 CppObjectType,BaseObjectType,3,&amp;gtk_container_focus&gt; focus;
</verb></tscreen>

From this declaration you can see the number of arguments required, 1.
The first template argument is type of the return, gint.  The arguments
that follow are the types of the argument.  In this case there is only
1 argument GtkDirectionType.  The rest of the arguments are internal
to specify various behaviors of the signal. 
Note that the above form for a signal callback function declaration is
only a general guide, as each signal has its own unique set of
different calling parameters.  There are many types of signals, but they all
share a similar form.

<!-- ----------------------------------------------------------------- -->
<sect1>Events
<p>
In addition to the signal mechanism described above, there are a set
of <em>events</em> that reflect the X event mechanism. Callbacks may
also be attached to these events, just like regular signals (in fact,
from the application programmer's point of view, there is no
difference). These events are:

<itemize>
<item> event
<item> button_press_event
<item> button_release_event
<item> motion_notify_event
<item> delete_event
<item> destroy_event
<item> expose_event
<item> key_press_event
<item> key_release_event
<item> enter_notify_event
<item> leave_notify_event
<item> configure_event
<item> focus_in_event
<item> focus_out_event
<item> map_event
<item> unmap_event
<item> property_notify_event
<item> selection_clear_event
<item> selection_request_event
<item> selection_notify_event
<item> proximity_in_event
<item> proximity_out_event
<item> drag_begin_event
<item> drag_request_event
<item> drag_end_event
<item> drop_enter_event
<item> drop_leave_event
<item> drop_data_available_event
<item> other_event
</itemize>

In order to connect a callback function to one of these events, you
use the connect_to_* functions, as described above, using one of the
above event names as the <tt/name/ parameter. The callbacks
for events have a slightly different form than that for signals:

<tscreen><verb>
void callback_func( GdkEvent  *event );
</verb></tscreen>

GdkEvent is a C <tt/union/ structure whose type will depend upon which
of the above events has occurred. In order for us to tell which event
has been issued each of the possible alternatives has a <tt/type/
parameter which reflects the event being issued. The other components
of the event structure will depend upon the type of the
event. Possible values for the type are:

<tscreen><verb>
  GDK_NOTHING
  GDK_DELETE
  GDK_DESTROY
  GDK_EXPOSE
  GDK_MOTION_NOTIFY
  GDK_BUTTON_PRESS
  GDK_2BUTTON_PRESS
  GDK_3BUTTON_PRESS
  GDK_BUTTON_RELEASE
  GDK_KEY_PRESS
  GDK_KEY_RELEASE
  GDK_ENTER_NOTIFY
  GDK_LEAVE_NOTIFY
  GDK_FOCUS_CHANGE
  GDK_CONFIGURE
  GDK_MAP
  GDK_UNMAP
  GDK_PROPERTY_NOTIFY
  GDK_SELECTION_CLEAR
  GDK_SELECTION_REQUEST
  GDK_SELECTION_NOTIFY
  GDK_PROXIMITY_IN
  GDK_PROXIMITY_OUT
  GDK_DRAG_BEGIN
  GDK_DRAG_REQUEST
  GDK_DROP_ENTER
  GDK_DROP_LEAVE
  GDK_DROP_DATA_AVAIL
  GDK_CLIENT_EVENT
  GDK_VISIBILITY_NOTIFY
  GDK_NO_EXPOSE
  GDK_OTHER_EVENT	/* Deprecated, use filters instead */
</verb></tscreen>

So, to connect a callback function to one of these events we would use:

<tscreen><verb>
button.button_press_event.connect( slot(&amp;button_press_callback) );
</verb></tscreen>

This assumes that <tt/button/ is a Gtk_Button widget. Now, when the
mouse is over the button and a mouse button is pressed, the function
<tt/button_press_callback/ will be called. This function may be
declared as:

<tscreen><verb>
static gint button_press_callback(GdkEventButton *event);
</verb></tscreen>

Note that we can declare the argument as type
<tt/GdkEventButton/ as we know what type of event will occur for this
function to be called.

The value returned from this function indicates whether the event
should be propagated further by the Gtk-- event handling
mechanism. Returning TRUE indicates that the event has been handled,
and that it should not propagate further. Returning FALSE continues
the normal event handling.  See the section on
<ref id="sec_Adv_Events_and_Signals"
name="Advanced Event and Signal Handling"> for more details on this
propagation process.

For details on the GdkEvent data types, see the appendix entitled
<ref id="sec_GDK_Event_Types" name="GDK Event Types">.

Note that there is currently no example here of connecting to events as
there are usually better solutions in GTK--.

<!-- ----------------------------------------------------------------- -->
<sect1>Overloading Member Functions
<p>
GTK-- offers an alternative to explicitly connecting signals and events:
the overloading of member functions. The functions you should override
are easily recognisable in the class reference as their name ends in <em>_impl</em>.
All events have corresponding member functions which are protected members
of Gtk_Widget. The member functions for all signals are defined on their
respective objects.

For example, to alter the behaviour of a window when a key is pressed
on it, override key_press_event_impl in the class:

<tscreen><verb>
class Window_Class : public Gtk_Window
{
protected:
  virtual int key_press_event_impl(GdkEventKey *Key_Event);
}

int Window_Class::key_press_event_impl(GdkEventKey *Key_Event)
{
  // Do your stuff with key presses

  // Call base class's member function in case it has to do anything
  Gtk_Window::key_press_event_impl(Key_Event);
}
</verb></tscreen>

Calling the base class's function is optional and depends upon what
you are trying to achieve.

For each object in a GTK-- program a decision has to be made whether
to derive a new class and overload member functions or just connect to
signals and events. A good rule of thumb is to override wherever
possible in a derived class. Just remember that overloading changes
the behaviour of the <em>class</em>, adding callbacks to signals and
events changes the behaviour of the <em>object</em>.

<!-- ----------------------------------------------------------------- -->
<sect1>Stepping Through Hello World
<p>
Now that we know the theory behind this, lets clarify by walking through 
the example <em>helloworld</em> program.

First, let's examine the main class :

<tscreen><verb>
class HelloWorld : public Gtk_Window
{
  Gtk_Button m_button;

public:
  HelloWorld();
  
  void hello();
  virtual int delete_event_handler(GdkEventAny *event);
  
};
</verb></tscreen>

It derives from Gtk_Window, and has a single Gtk_Button as a
member. The class ctor will connect the various signals to their
respective callbacks, put the button in the window and show the
button and window.

Here is the callback method that will be called when the button is
"clicked". In the next example we will add a data argument to
tell us which button was pressed.

<tscreen><verb>
void HelloWorld::hello()
{
  cout << "Hello World" << endl;
}
</verb></tscreen>

The next event is handled by overriding the member function of the
base class. The delete_event_impl function is called when the
window manager sends this event to the application. We have a choice
here as to what to do about this event. We can ignore it, make
some sort of response, or simply quit the application.

The value you return in this function lets Gtk-- know what action to
take.  By returning true, we let it know that we don't want to have
the "destroy" signal emitted, keeping our application running. By
returning false, we ask that "destroy" is emitted, which in turn will
call our "destroy" signal handler.

<tscreen><verb>
int HelloWorld::delete_event_impl(GdkEventAny *event)
{
  cout << "delete event occured" << endl;
  return true; 
}
</verb></tscreen>

Here is another callback method which causes the program to quit by
calling quit() on the global Gtk_Main object. This function tells
Gtk-- that it is to exit from the internal GTK+ gtk_main() loop when
control is returned to it.

<tscreen><verb>
void destroy_handler()
{
  Gtk_Main::quit();
}
</verb></tscreen>

I assume you know about the main() function... yes, as with other
applications, all Gtk-- applications will also have one of these.

<tscreen><verb>
int main (int argc, char *argv[])
{
</verb></tscreen>

Next we shall initialize our kit by creating an instance of the 
Gtk_Main object.

<tscreen><verb>
  Gtk_Main kit(&amp;argc, &amp;argv);
</verb></tscreen>

We then create a HelloWorld object, our derived window :
<tscreen><verb>
  HelloWorld helloworld;
</verb></tscreen>

Let's now examine the HelloWorld ctor.

<tscreen><verb>
HelloWorld::HelloWorld()
  : Gtk_Window(GTK_WINDOW_TOPLEVEL),
    m_button("Hello World")
{
</verb></tscreen>

The parent (Gtk_Window) is constructed as being a top level window
(that is, a stand-alone, independant window).

Then, the m_button data member is created. It will have the label
"Hello World" on it when displayed.

Here is an example of connecting a signal handler to an object, the
window in this case. Here the "destroy" signal is caught. This signal is
emitted when we use the window manager to kill the window (and we
return false in the "delete_event" handler), or when we use the
Gtk_Widget::destroy() call.  By setting this up, we handle both cases
with a single function. The destroy_handler() function defined above
is called, which quits Gtk-- for us.

<tscreen><verb>
  destroy.connect( slot(&amp;destroy_handler) );
</verb></tscreen>

This next method is used to set an attribute of a container object.
It sets the window so it has a blank area along the inside of
it 10 pixels wide where no widgets will go. There are other similar
functions which we will look at in the section on
<ref id="sec_setting_widget_attributes" name="Setting Widget Attributes">

<tscreen><verb>
  set_border_width(10);
</verb></tscreen>

Next, we take the button, and make it do something useful. We attach
a signal handler to it so when it emits the "clicked" signal, our
hello() method is called. The data is ignored, so we simply pass in
NULL to the hello() callback function. Obviously, the "clicked" signal
is emitted when we click the button with our mouse pointer.

<tscreen><verb>
  m_button.clicked.connect( slot(this, &amp;HelloWorld::hello) );
</verb></tscreen>

We are also going to use this button to exit our program. A signal can
be connected to another signal, so here we will connect the button's
"clicked" signal to our window's "destroy" signal.

<tscreen><verb>
  m_button.clicked.connect(destroy.slot());
</verb></tscreen>

The next statement  is a packing call, which will be explained in depth
later on. It simply tells Gtk-- that the button is to be placed in the
window where it will be displayed. Note that a Gtk-- container can only
contain one widget. There are other widgets, that are described later,
which are designed to layout multiple widgets in various ways.
 
<tscreen><verb>
  add(m_button);
</verb></tscreen>

Now we have everything set up the way we want it to be. With all the
signal handlers in place, and the button placed in the window where it
should be, we ask Gtk-- to "show" the widgets on the screen. The window
widget is shown last so the whole window will pop up at once rather
than seeing the window pop up, and then the button form inside of
it. Although with such a simple example, you'd never notice.

<tscreen><verb>
  m_button.show();

  show();
</verb></tscreen>

This could have been shortened in a single call :

<tscreen><verb>
  show_all();
</verb></tscreen>

which would have shown the window, and its children recursively.

This ends the HelloWindow ctor. Back again to the main() function, all 
that remains is to call 

<tscreen><verb>
  myapp.run();
</verb></tscreen>

which waits for events to come from the X server and will call on the
widgets to emit signals when these events come.

And the final return. Control returns here after Gtk_Main::quit() is called.

<tscreen><verb>
    return 0;
</verb></tscreen>

Now, when we click the mouse button on a Gtk-- button, the widget
emits a "clicked" signal. In order for us to use this information, our
program sets up a signal handler to catch that signal, which
dispatches the function of our choice. When the button
we created is "clicked", the HelloWorld::hello() method is called.
Then the next handler for this signal is
called, the destroy() method on the window widget. This
causes the window to emit the "destroy" signal, which is caught, and
calls our destroy_handler() callback function, which exits
Gtk--.

Another course of events, is to use the window manager to kill the
window. This will cause the "delete_event" to be emitted. This will
call our "delete_event_handler" method. If we return true here, the
window will be left as is and nothing will happen. Returning false
will cause Gtk-- to emit the "destroy" signal, which calls
the "destroy_handler()" callback, exiting Gtk--.

Note that these signals are not the same as the Unix system signals,
and are not implemented using them, although the terminology is almost
identical.

