<sect> Miscellaneous Widgets
<!-- ***************************************************************** -->

<!-- ----------------------------------------------------------------- -->
<sect1> Labels
<p>
Labels are used a lot in Gtk--, and are relatively simple. Labels emit
no signals as they do not have an associated X window. If you need to
catch signals, or do clipping, use the <ref id="sec_EventBox"
name="EventBox"> widget.

The ctors for Gtk_Label are 

<tscreen><verb>
Gtk_Label(const _gtk_string &amp;label = 0);
</verb></tscreen>

The sole argument is the string you wish the label to display.

<tt//_gtk_string/ is a special Gtk-- version of the C++ string, which
can also be a NULL ptr. It's internal to Gtk--, all you need to know
is that you can set pass 0 as a _gtk_string argument value whenever
you would want to pass a NULL ptr (something you can't do with a
standard C++ string). The analogy with a C function taking a char*
argument should make things clearer :

<tscreen><verb>
void a_c_func(char *arg);
...
a_c_func(0);

void a_cxx_func(_gtk_string &amp;arg);
...
a_cxx_func(0);
</verb></tscreen>


To change the label's text after creation, use the method:

<tscreen><verb>
void Gtk_Label::set_text(const string &amp;str);
</verb></tscreen>

Where the argument is the new string.

The space needed for the new string will be automatically adjusted if
needed. You can produce multi-line labels by putting line breaks in
the label string.

To retrieve the current string, use:

<tscreen><verb>
string Gtk_Label::get_text();
</verb></tscreen>

The label text can be justified using:

<tscreen><verb>
void Gtk_Label::set_justify(GtkJustification  jtype );
</verb></tscreen>

Values for <tt/jtype/ are:
<itemize>
<item> GTK_JUSTIFY_LEFT
<item> GTK_JUSTIFY_RIGHT
<item> GTK_JUSTIFY_CENTER (the default)
<item> GTK_JUSTIFY_FILL
</itemize>

The label widget is also capable of line wrapping the text
automatically. This can be activated using:

<tscreen><verb>
void Gtk_Label::set_line_wrap (bool wrap);
</verb></tscreen>

The <//wrap/ argument takes a TRUE or FALSE value.

If you want your label underlined, then you can set a pattern on the
label:

<tscreen><verb>
void Gtk_Label::set_pattern   (const _gtk_string &amp;pattern);
</verb></tscreen>

The pattern argument indicates how the underlining should look. It
consists of a string of underscore and space characters. An underscore
indicates that the corresponding character in the label should be
underlined. For example, the string <verb/"__     __"/ would underline the
first two characters and eigth and ninth characters.

Below is a short example to illustrate these functions. This example
makes use of the Frame widget to better demonstrate the label
styles. You can ignore this for now as the <ref id="sec_Frames"
name="Frame"> widget is explained later on.

<tscreen><verb>
example_include(label/label.cc)
</verb></tscreen>

<!-- ----------------------------------------------------------------- -->
<sect1> Arrows
<p>
The Arrow widget draws an arrowhead, facing in a number of possible
directions and having a number of possible styles. It can be very
useful when placed on a button in many applications.

The ctor for Gtk_Arrow is

<tscreen><verb>
Gtk_Arrow( GtkArrowType   arrow_type,
           GtkShadowType  shadow_type );
</verb></tscreen>

and you can manipulate it with

<tscreen><verb>
void Gtk_Arrow::set(GtkArrowType   arrow_type,
                    GtkShadowType  shadow_type );
</verb></tscreen>

which allows the arrow and shadow type which were specified on
construction to be altered. The <tt/arrow_type/ argument may take one
of the following values:

<itemize>
<item> GTK_ARROW_UP
<item> GTK_ARROW_DOWN
<item> GTK_ARROW_LEFT
<item> GTK_ARROW_RIGHT
</itemize>

These values obviously indicate the direction in which the arrow will
point. The <tt/shadow_type/ argument may take one of these values:

<itemize>
<item> GTK_SHADOW_IN
<item> GTK_SHADOW_OUT (the default)
<item> GTK_SHADOW_ETCHED_IN
<item> GTK_SHADOW_ETCHED_OUT
</itemize>

Here's a brief example to illustrate their use.

<tscreen><verb>
example_include(arrow/arrow.cc)
</verb></tscreen>

<!-- ----------------------------------------------------------------- -->
<sect1>The Tooltips Widget
<p>
These are the little text strings that pop up when you leave your
pointer over a button or other widget for a few seconds. They are easy
to use, so I will just explain them without giving an example. If you
want to see some code, take a look at the testgtk.c program
distributed with Gtk--.

Widgets that do not receieve events (widgets that do not have their
own window) will not work with tooltips.

The ctor is extremely simple :

<tscreen><verb>
Gtk_Tooltips();
</verb></tscreen>

Once you have created a new tooltip, and the widget you wish to use it
on, simply use this call to set it:

<tscreen><verb>
void Gtk_Tooltips::set_tip(const Gtk_Widget   &amp;widget,
                           const _gtk_string  &amp;tip_text,
                           const _gtk_string  &amp;tip_private );
</verb></tscreen>

The first argument is the widget you wish to have this tooltip pop up
for, and the text you wish it to say. The last argument is a text
string that can be used as an identifier when using Gtk_TipsQuery to
implement context sensitive help. For now, you can set it to 0.

<!-- TODO: sort out what how to do the context sensitive help -->

Here's a short example:

<tscreen><verb>
Gtk_Tooltips tooltips;
Gtk_Widget button("button1");
.
.
.
tooltips.set_tip (button, "This is button 1", 0);
</verb></tscreen>

There are other calls that can be used with tooltips. I will just list
them with a brief description of what they do.

<tscreen><verb>
void Gtk_Tooltips::enable();
</verb></tscreen>

Enable a disabled set of tooltips.

<tscreen><verb>
void Gtk_Tooltips::disable();
</verb></tscreen>

Disable an enabled set of tooltips.

<tscreen><verb>
void Gtk_Tooltips::set_delay(gint         delay );

</verb></tscreen>

Sets how many milliseconds you have to hold your pointer over the
widget before the tooltip will pop up. The default is 500
milliseconds (half a second).

<tscreen><verb>
void gtk_tooltips::set_colors(const Gdk_Color    &amp;background,
                              const Gdk_Color    &amp;foreground );
</verb></tscreen>

Set the foreground and background color of the tooltips.

And that's all the methods associated with tooltips. More than
you'll ever want to know :-)

<!-- ----------------------------------------------------------------- -->
<sect1> Progress Bars <label id="sec_ProgressBar">
<p>
Progress bars are used to show the status of an operation. They are
pretty easy to use, as you will see with the code below. But first
lets start out with the calls to create a new progress bar.

There are two ways to create a progress bar, one simple that takes no
arguments, and one that takes a Gtk_Adjustment object as an
argument. If the former is used, the progress bar creates its own
adjustment object.

<tscreen><verb>
Gtk_ProgressBar();

Gtk_ProgressBar(GtkAdjustment &amp;adjustment);
</verb></tscreen>

The second method has the advantage that we can use the adjustment
object to specify our own range parameters for the progress bar.

The adjustment of a progress object can be changed dynamically using:

<tscreen><verb>
void Gtk_Progress::set_adjustment(GtkAdjustment &amp;adjustment);
</verb></tscreen>

(Gtk_ProgressBar derives from Gtk_Progress).

Now that the progress bar has been created we can use it.

<tscreen><verb>
void Gtk_Progress::bar_update(gfloat          percentage );
</verb></tscreen>

The first argument is the progress bar you wish to operate on, and the
second argument is the amount 'completed', meaning the amount the
progress bar has been filled from 0-100%. This is passed to the
method as a real number ranging from 0 to 1.

The progress bar can display its value in different ways, and inform
the user of its current value and its range.

A progress bar may be set to one of a number of orientations using the
method

<tscreen><verb>
void Gtk_ProgressBar::set_orientation( GtkProgressBarOrientation orientation );
</verb></tscreen>

Where the <tt/orientation/ argument may take one of the following
values to indicate the direction in which the progress bar moves:

<itemize>
<item> GTK_PROGRESS_LEFT_TO_RIGHT
<item> GTK_PROGRESS_RIGHT_TO_LEFT
<item> GTK_PROGRESS_BOTTOM_TO_TOP
<item> GTK_PROGRESS_TOP_TO_BOTTOM
</itemize>

When used as a measure of how far a process has progressed, the
Gtk_ProgressBar can be set to display its value in either a continuous
or discrete mode. In continuous mode, the progress bar is updated for
each value. In discrete mode, the progress bar is updated in a number
of discrete blocks. The number of blocks is also configurable.

The style of a progress bar can be set using the following method.

<tscreen><verb>
void Gtk_ProgressBar::set_bar_style( GtkProgressBarStyle  style );
</verb></tscreen>

The <tt/style/ parameter can take one of two values:

<itemize>
<item>GTK_PROGRESS_CONTINUOUS
<item>GTK_PROGRESS_DISCRETE
</itemize>

The number of discrete blocks can be set by calling

<tscreen><verb>
void Gtk_ProgressBar::set_discrete_blocks( guint           blocks );
</verb></tscreen>

As well as indicating the amount of progress that has occured, the
progress bar may be set to just indicate that there is some
activity. This can be useful in situations where progress cannot be
measured against a value range. Activity mode is not effected by the
bar style that is described above, and overrides it.This mode is
selected by the following method.

<tscreen><verb>
void Gtk_Progress::set_activity_mode(guint        activity_mode );
</verb></tscreen>

The step size of the activity indicator, and the number of blocks are
set using the following methods.

<tscreen><verb>
void Gtk_ProgressBar::set_activity_step(guint           step );

void Gtk_ProgressBar::set_activity_blocks(guint           blocks );
</verb></tscreen>

When in continuous mode, the progress bar can also display a
configurable text string within its trough, using the following
method.

<tscreen><verb>
void Gtk_Progress::set_format_string(const string       &amp;format);
</verb></tscreen>

The <tt/format/ argument is similiar to one that would be used in a C
<tt/printf/ statement. The following directives may be used within the
format string:

<itemize>
<item> %p - percentage
<item> %v - value
<item> %l - lower range value
<item> %u - upper range value
</itemize>

The displaying of this text string can be toggled using:

<tscreen><verb>
void Gtk_Progress::set_show_text(bool         show_text );
</verb></tscreen>

The <tt/show_text/ argument is a boolean TRUE/FALSE value. The
appearance of the text can be modified further using:

<tscreen><verb>
void Gtk_Progress::set_text_alignment(gfloat         x_align,
                                      gfloat         y_align );
</verb></tscreen>

The <tt/x_align/ and <tt/y_align/ arguments take a value between 0.0
and 1.0. Their value indicates the position of the text string within
the trough. Values of 0.0 for both would place the string in the top
left hand corner; values of 0.5 (the default) centres the text, and
values of 1.0 places the text in the lower right hand corner.

The current text setting of a progress object can be retrieved using
the current or a specified adjustment value using the following two
methods. They return the formatted string that would be displayed
within the trough.

<tscreen><verb>
string Gtk_Progress::get_current_text();

string Gtk_Progress::get_text_from_value(gfloat       value );
</verb></tscreen>

There is yet another way to change the range and value of a progress
object using the following function:

<tscreen><verb>
void Gtk_Progress::configure(gfloat        value,
                             gfloat        min,
                             gfloat        max );
</verb></tscreen>

This function provides quite a simple interface to the range and value
of a progress object.

The remaining functions can be used to get and set the current value
of a progess object in various types and formats:

<tscreen><verb>
void Gtk_Progress::set_percentage(gfloat       percentage );

void Gtk_Progress::set_value(gfloat       value );

gfloat Gtk_Progress::get_value();

gfloat Gtk_Progress::get_current_percentage();

gfloat Gtk_Progress::get_percentage_from_value(gfloat       value );
</verb></tscreen>

These functions are pretty self explanatory. The last function uses
the adjustment of the specified progess object to compute the
percentage value of the given range value.

Progress Bars are usually used with timeouts or other such functions
(see section on <ref id="sec_timeouts" name="Timeouts, I/O and Idle
Functions">) to give the illusion of multitasking. All will employ the
Gtk_Progress::bar_update function in the same manner.

Here is an example of the progress bar, updated using timeouts. This
code also shows you how to reset the Progress Bar.

<tscreen><verb>
example_include(progressbar/progressbar.cc)
</verb></tscreen>

<!-- ----------------------------------------------------------------- -->
<sect1> Dialogs
<p>
The Dialog widget is very simple, and is actually just a window with a
few things pre-packed into it for you. It simply creates a window, and
then packs a vbox into the top, then a separator, and then an hbox for
the "action_area".

The Dialog widget can be used for pop-up messages to the user, and
other similar tasks. It is really basic, and there is only one
ctor for the dialog box, which is:

<tscreen><verb>
Gtk_Dialog( );
</verb></tscreen>

You could pack a button in the action_area by doing something like this:

<tscreen><verb>
    Gtk_Dialog dialog;
    Gtk_Button button("click me");
    Gtk_HBox *dialogHBox = dialog.get_action_area();
    dialogHBox->pack_start (button, TRUE, TRUE, 0);
    button.show();
</verb></tscreen>

And you could add to the vbox area by packing, for instance, a label 
in it, try something like this:

<tscreen><verb>
    Gtk_Label label("Dialogs are groovy");
    Gtk_VBox *vbox = dialog.get_vbox();
    vbox->pack_start (label, TRUE, TRUE, 0);
    label.show();
</verb></tscreen>

As an example in using the dialog box, you could put two buttons in
the action_area, a Cancel button and an Ok button, and a label in the
vbox area, asking the user a question or giving an error etc. Then
you could attach a different signal to each of the buttons and perform
the operation the user selects.

If the simple functionality provided by the default vertical and
horizontal boxes in the two areas does't give you enough control for
your application, then you can simply pack another layout widget into
the boxes provided. For example, you could pack a table into the
vertical box.

<!-- ----------------------------------------------------------------- -->
<sect1> Pixmaps <label id="sec_Pixmaps">
<p>
Pixmaps are data structures that contain pictures. These pictures can
be used in various places, but most visibly as icons on the X-Windows
desktop, or as cursors.

A pixmap which only has 2 colors is called a bitmap, and there are a
few additional routines for handling this common special case.

To understand pixmaps, it would help to understand how X-windows
works. Under X-windows, applications do not need to be running on the
same computer that is interacting with the user. Instead, the various
applications, called "clients", all communicate with a program which
displays the graphics and handles the keyboard and mouse. This
program which interacts directly with the user is called a "display
server" or "X server." Since the communication might take place over
a network, it's important to keep some information with the X server.
Pixmaps, for example, are stored in the memory of the X server. This
means that once pixmap values are set, they don't need to keep getting
transmitted over the network; instead a command is sent to "display
pixmap number XYZ here." Even if you aren't using X-windows with Gtk--
currently, using constructs such as Pixmaps will make your programs
work acceptably under X-windows.

There are several ways to create pixmaps in Gtk--. The simplest one is 
to use the following ctors :

<tscreen><verb>
Gtk_Pixmap(Gtk_Widget &amp;w, const _gtk_string &amp;xpmfilename);
Gtk_Pixmap(Gtk_Widget &amp;w, const gchar *const *data);
</verb></tscreen>

In both cases, the Gtk_Widget which is passed as 1st argument should
be the one in which the pixmap will eventually be displayed (e.g. a
container, like a Gtk_Window or Gtk_Button). It's called the reference 
widget.

The <tt/xpmfilename/ parameter of the 1st ctor is, as expected, the
name of an XPM file.

The <tt/data/ parameter of the 2nd ctor is a pointer to raw XPM
data. This data must not be freed until the reference widget is
realized.

Another more complicated one is to first build a Gdk_Pixmap structure
using routines from the Gdk-- layer. Pixmaps can either be created
from in-memory data, or from data read from a file. We'll go through
each of the calls to create a pixmap.

<tscreen><verb>
Gdk_Bitmap( Gdk_Window &amp;window,
            gchar     *data,
            gint       width,
            gint       height );
</verb></tscreen>

This routine is used to create a single-plane pixmap (2 colors) from
data in memory. Each bit of the data represents whether that pixel is
off or on. Width and height are in pixels. The Gdk_Window is the
current window, since a pixmap resources are meaningful only in the
context of the screen where it is to be displayed.

<tscreen><verb>
Gdk_Pixmap ( GdkWindow *window,
             gchar     *data,
             gint       width,
             gint       height,
             gint       depth,
             const Gdk_Color  &amp;fg,
             const Gdk_Color  &amp;bg );
</verb></tscreen>

This is used to create a pixmap of the given depth (number of colors) from
the bitmap data specified. <tt/fg/ and <tt/bg/ are the foreground and
background color to use.

<tscreen><verb>
Gdk_Pixmap( Gdk_Window   &amp;window,
            Gdk_Bitmap   &amp;mask,
            const Gdk_Color    &amp;transparent_color,
            const string &amp;filename );
</verb></tscreen>

XPM format is a readable pixmap representation for the X Window
System. It is widely used and many different utilities are available
for creating image files in this format. The file specified by
filename must contain an image in that format and it is loaded into
the pixmap structure. The mask specifies which bits of the pixmap are
opaque. All other bits are colored using the color specified by
transparent_color. An example using this follows below.

<tscreen><verb>
Gdk_Pixmap::create_from_xpm_d( const Gdk_Drawable &amp;drawable,
                               Gdk_Bitmap &amp;mask,
                               const Gdk_Color &amp;transparent_color,
                               gchar *const *data );
</verb></tscreen>

Small images can be incorporated into a program as data in the XPM
format. A pixmap is created using this data, instead of reading it
from a file. An example of such data is

<tscreen><verb>
/* XPM */
static const char *const xpm_data[] = {
"16 16 3 1",
"       c None",
".      c #000000000000",
"X      c #FFFFFFFFFFFF",
"                ",
"   ......       ",
"   .XXX.X.      ",
"   .XXX.XX.     ",
"   .XXX.XXX.    ",
"   .XXX.....    ",
"   .XXXXXXX.    ",
"   .XXXXXXX.    ",
"   .XXXXXXX.    ",
"   .XXXXXXX.    ",
"   .XXXXXXX.    ",
"   .XXXXXXX.    ",
"   .XXXXXXX.    ",
"   .........    ",
"                ",
"                "};
</verb></tscreen>

When we're done using a pixmap and not likely to reuse it again soon,
it is a good idea to delete it. Pixmaps should be considered a
precious resource.

Once we've created a pixmap, we can now display it as a Gtk--
widget. The appropriate Gtk_Pixmap ctor is

<tscreen><verb>
Gtk_Pixmap(const Gdk_Pixmap &amp;pixmap,
           const Gdk_Bitmap &amp;mask);
</verb></tscreen>

The other pixmap widget calls are

<tscreen><verb>
void  Gtk_Pixmap::set(const Gdk_Pixmap &amp;newpixmap,
	              const Gdk_Bitmap &amp;mask );

void  Gtk_Pixmap::get(Gdk_Pixmap &amp;pixmap,
	              Gdk_Bitmap &amp;mask);
</verb></tscreen>

Gtk_Pixmap::set is used to change the pixmap that the widget is
currently managing. newpixmap is the pixmap created using GDK.

The following is an example of using a pixmap in a button.

<tscreen><verb>
example_include(pixmap/pixmap.cc)
</verb></tscreen>

To load a file from an XPM data file called icon0.xpm in the current
directory, we would have created the pixmap thus

<tscreen><verb>
    // load a pixmap from a file
    Gtk_Pixmap pixmap(window, "./icon0.xpm" );
    window.add(pixmap);
</verb></tscreen>

A disadvantage of using pixmaps is that the displayed object is always
rectangular, regardless of the image. We would like to create desktops
and applications with icons that have more natural shapes. For
example, for a game interface, we would like to have round buttons to
push. The way to do this is using shaped windows.

A shaped window is simply a pixmap where the background pixels are
transparent. This way, when the background image is multi-colored, we
don't overwrite it with a rectangular, non-matching border around our
icon. The following example displays a full wheelbarrow image on the
desktop.

<tscreen><verb>
example_include(wheelbarrow/wheelbarrow.cc)
</verb></tscreen>

To make the wheelbarrow image sensitive, we could attach the button
press event signal to make it do something. The following few lines
would make the picture sensitive to a mouse button being pressed which
makes the application terminate.

<tscreen><verb>
    window.set_events(window.get_events() | GDK_BUTTON_PRESS_MASK );
    window.button_press_event.connect( slot(&amp;close_application) );
</verb></tscreen>

<!-- ----------------------------------------------------------------- -->
<sect1>Rulers
<p>
Ruler widgets are used to indicate the location of the mouse pointer
in a given window. A window can have a vertical ruler spanning across
the width and a horizontal ruler spanning down the height. A small
triangular indicator on the ruler shows the exact location of the
pointer relative to the ruler.

Horizontal and vertical rulers have one very simple ctor :

<tscreen><verb>
Gtk_HRuler();    // horizontal ruler

Gtk_VRuler();    // vertical ruler
</verb></tscreen>

Once a ruler is created, we can define the unit of measurement. Units
of measure for rulers can be GTK_PIXELS, GTK_INCHES or
GTK_CENTIMETERS. This is set using

<tscreen><verb>
void Gtk_Ruler::set_metric(GtkMetricType  metric);
</verb></tscreen>

The default measure is GTK_PIXELS.

<tscreen><verb>
    ruler.set_metric(GTK_PIXELS);
</verb></tscreen>

Other important characteristics of a ruler are how to mark the units
of scale and where the position indicator is initially placed. These
are set for a ruler using

<tscreen><verb>
void Gtk_Ruler::set_range(gfloat    lower,
                          gfloat    upper,
                          gfloat    position,
                          gfloat    max_size );
</verb></tscreen>

The lower and upper arguments define the extent of the ruler, and
max_size is the largest possible number that will be displayed.
Position defines the initial position of the pointer indicator within
the ruler.

A vertical ruler can span an 800 pixel wide window thus

<tscreen><verb>
    vruler.set_range(0, 800, 0, 800);
</verb></tscreen>

The markings displayed on the ruler will be from 0 to 800, with a
number for every 100 pixels. If instead we wanted the ruler to range
from 7 to 16, we would code

<tscreen><verb>
    vruler.set_range(7, 16, 0, 20);
</verb></tscreen>

The indicator on the ruler is a small triangular mark that indicates
the position of the pointer relative to the ruler. If the ruler is
used to follow the mouse pointer, the motion_notify_event signal
should be connected to the motion_notify_event method of the ruler.
To follow all mouse movements within a window area, we would use

<tscreen><verb>
!!!!!!!!!!
#define EVENT_METHOD(i, x) GTK_WIDGET_CLASS(GTK_OBJECT(i)->klass)->x

    gtk_signal_connect_object( GTK_OBJECT(area), "motion_notify_event",
           (GtkSignalFunc)EVENT_METHOD(ruler, motion_notify_event),
           GTK_OBJECT(ruler) );
</verb></tscreen>

The following example creates a drawing area with a horizontal ruler
above it and a vertical ruler to the left of it. The size of the
drawing area is 600 pixels wide by 400 pixels high. The horizontal
ruler spans from 7 to 13 with a mark every 100 pixels, while the
vertical ruler spans from 0 to 400 with a mark every 100 pixels.
Placement of the drawing area and the rulers is done using a table.

<tscreen><verb>
example_include(rulers/rulers.cc)
</verb></tscreen>

<!-- ----------------------------------------------------------------- -->
<sect1>Statusbars
<p>
Statusbars are simple widgets used to display a text message. They
keep a stack of the messages pushed onto them, so that popping the
current message will re-display the previous text message.

In order to allow different parts of an application to use the same
statusbar to display messages, the statusbar widget issues Context
Identifiers which are used to identify different 'users'. The message
on top of the stack is the one displayed, no matter what context it is
in. Messages are stacked in last-in-first-out order, not context
identifier order.

Gtk_Statusbar has a very simple ctor

<tscreen><verb>
Gtk_Statusbar();
</verb></tscreen>

A new Context Identifier is requested using a call to the following 
method with a short textual description of the context:

<tscreen><verb>
guint Gtk_Statusbar::get_context_id(const _gtk_string &amp;context_description);
</verb></tscreen>

There are three methods that can operate on statusbars:

<tscreen><verb>
guint Gtk_Statusbar::push(guint         context_id,
                          const _gtk_string &amp;text );

void Gtk_Statusbar::pop(guint         context_id );

void Gtk_Statusbar::remove(guint         context_id,
                           guint         message_id ); 
</verb></tscreen>

The first, Gtk_Statusbar::push, is used to add a new message to the
statusbar.  It returns a Message Identifier, which can be passed later
to the function Gtk_Statusbar::remove to remove the message with the
given Message and Context Identifiers from the statusbar's stack.

The method Gtk_Statusbar::pop removes the message highest in the
stack with the given Context Identifier.

The following example creates a statusbar and two buttons, one for
pushing items onto the statusbar, and one for popping the last item
back off.

<tscreen><verb>
example_include(statusbar/statusbar.cc)
</verb></tscreen>

<!-- ----------------------------------------------------------------- -->
<sect1>Text Entries
<p>
The Entry widget allows text to be typed and displayed in a single line
text box. The text may be set with function calls that allow new text
to replace, prepend or append the current contents of the Entry widget.

There are two ctors for Gtk_Entry widgets:

<tscreen><verb>
Gtk_Entry();

Gtk_Entry( guint16 max );
</verb></tscreen>

The first just creates a new Entry widget, whilst the second creates a
new Entry and sets a limit on the length of the text within the Entry.

There are several functions for altering the text which is currently
within the Entry widget.

<tscreen><verb>
void Gtk_Entry::set_text(const string &amp;text);

void Gtk_Entry::append_text(const string &amp;text);

void Gtk_Entry::prepend_text(const string &amp;text);
</verb></tscreen>

The method Gtk_Entry::set_text sets the contents of the Entry widget,
replacing the current contents. The methods Gtk_Entry::append_text
and Gtk_Entry::prepend_text allow the current contents to be appended
and prepended to.

The next function allows the current insertion point to be set.

<tscreen><verb>
void Gtk_Entry::set_position(gint      position);
</verb></tscreen>

The contents of the Entry can be retrieved by using a call to the
following method. This is useful in the callback functions described below.

<tscreen><verb>
string Gtk_Entry::get_text();
</verb></tscreen>

If we don't want the contents of the Entry to be changed by someone typing
into it, we can change its editable state.

<tscreen><verb>
void Gtk_Entry::set_editable(bool  editable);
</verb></tscreen>

The function above allows us to toggle the editable state of the
Entry widget by passing in a TRUE or FALSE value for the <tt/editable/
argument.

If we are using the Entry where we don't want the text entered to be
visible, for example when a password is being entered, we can use the
following method, which also takes a boolean flag.

<tscreen><verb>
void Gtk_Entry::set_visibility(bool  visible);
</verb></tscreen>

A region of the text may be set as selected by using the following
function. This would most often be used after setting some default
text in an Entry, making it easy for the user to remove it.

<tscreen><verb>
void Gtk_Entry::select_region(gint      start,
                              gint      end );
</verb></tscreen>

If we want to catch when the user has entered text, we can connect to
the <tt/activate/ or <tt/changed/ signal. Activate is raised when the
user hits the enter key within the Entry widget. Changed is raised
when the text changes at all, e.g. for every character entered or
removed.

The following code is an example of using an Entry widget.

<tscreen><verb>
#include <iostream>
#include <gtk--.h>

// Gtk-- version of the "entry" example from the gtk+ tutorial

class Entry : public Gtk_Window
{
  Gtk_HBox m_hbox;
  Gtk_VBox m_vbox;
  Gtk_Entry m_entry;
  Gtk_Button m_bClose;
  Gtk_CheckButton m_cbEditable, m_cbVisible;

  void toggleEditable() { m_entry.set_editable(m_cbEditable.get_active()); }
  
  void toggleVisibility() { m_entry.set_visibility(m_cbVisible.get_active()); }
  void enterCallback();

  gint delete_event_impl(GdkEventAny*) { 
    Gtk_Main::instance()->quit(); return 0; 
  }
  
public:
  Entry();
};

Entry::Entry() :
  m_hbox(false, 0),
  m_vbox(false, 0),
  m_entry(50),
  m_bClose("Close"),
  m_cbEditable("Editable"),
  m_cbVisible("Visible")
{

  set_usize(200, 100);
  set_title("Gtk-- Entry");
  
  add(&amp;m_vbox);
  
  connect_to_method(m_entry.activate, this, &amp;enterCallback);
  m_entry.set_text("hello");
  m_entry.append_text(" world");
  m_entry.select_region(0, m_entry.get_text_length());
  m_vbox.pack_start(m_entry);
  
  // Note that add() can also be used instead of pack_xxx()
  m_vbox.add(&amp;m_hbox);

  m_hbox.pack_start(m_cbEditable);
  connect_to_method(m_cbEditable.toggled, this, &amp;toggleEditable);
  m_cbEditable.set_active(true);

  m_hbox.pack_start(m_cbVisible);
  connect_to_method(m_cbVisible.toggled, this, &amp;toggleVisibility);
  m_cbVisible.set_active(true);

  connect_to_method(m_bClose.clicked, Gtk_Main::instance(), &amp;Gtk_Main::quit);
  m_vbox.pack_start(m_bClose);
  m_bClose.set_flags(GTK_CAN_DEFAULT);
  m_bClose.grab_default();
  
  show_all();
}

void
Entry::enterCallback()
{
  cout << "Entry contents : " << m_entry.get_text() << endl;
}

  

// Gtk-- version of the "entry" example from the gtk+ tutorial

int main (int argc, char *argv[])
{
  Gtk_Main myapp(argc, argv);

  Entry entry;

  myapp.run();
  return 0;
}
/* example-end */
</verb></tscreen>

<!-- ----------------------------------------------------------------- -->
<sect1>Spin Buttons
<p>
The Spin Button widget is generally used to allow the user to select a
value from a range of numeric values. It consists of a text
entry box with up and down arrow buttons attached to the
side. Selecting one of the buttons causes the value to 'spin' up and
down the range of possible values. The entry box may also be edited
directly to enter a specific value.

The Spin Button allows the value to have zero or a number of decimal
places and to be incremented/decremented in configurable steps. The
action of holding down one of the buttons optionally result in an
acceleration of change in the value according to how long it is
depressed.

The Spin Button uses an <ref id="sec_Adjustment" name="Adjustment">
object to hold information about the range of values that the spin
button can take. This makes for a powerful Spin Button widget.

Recall that an adjustment widget is created with the following
ctor, which illustrates the information that it holds:

<tscreen><verb>
Gtk_Adjustment( gfloat value,
                gfloat lower,
                gfloat upper,
                gfloat step_increment,
                gfloat page_increment,
                gfloat page_size );
</verb></tscreen>

These attributes of an Adjustment are used by the Spin Button in the
following way:

<itemize>
<item> <tt/value/: initial value for the Spin Button
<item> <tt/lower/: lower range value
<item> <tt/upper/: upper range value
<item> <tt/step_increment/: value to increment/decrement when pressing
mouse button 1 on a button
<item> <tt/page_increment/: value to increment/decrement when pressing
mouse button 2 on a button
<item> <tt/page_size/: unused
</itemize>

Additionally, mouse button 3 can be used to jump directly to the
<tt/upper/ or <tt/lower/ values when used to select one of the
buttons. Lets look at the Gtk_SpinButton ctor:

<tscreen><verb>
Gtk_SpinButton( GtkAdjustment  &amp;adjustment,
                gfloat         climb_rate,
                guint          digits );
</verb></tscreen>

The <tt/climb_rate/ argument take a value between 0.0 and 1.0 and
indicates the amount of acceleration that the Spin Button has. The
<tt/digits/ argument specifies the number of decimal places to which
the value will be displayed.

The adjustment can be set and retrieved independantly using the
following two functions:

<tscreen><verb>
void Gtk_SpinButton::set_adjustment(const Gtk_Adjustment &amp;adjustment);
!!!!!!!!!!
Gtk_Adjustment* Gtk_SpinButton::get_adjustment();
</verb></tscreen>

The number of decimal places can also be altered using:

<tscreen><verb>
void Gtk_SpinButton::set_digits(guint          digits) ;
</verb></tscreen>

The value that a Spin Button is currently displaying can be changed
using the following function:

<tscreen><verb>
void Gtk_SpinButton::set_value(gfloat         value);
</verb></tscreen>

The current value of a Spin Button can be retrieved as either a
floating point or integer value with the following functions:

<tscreen><verb>
gfloat Gtk_SpinButton::get_value_as_float();

gint Gtk_SpinButton::get_value_as_int();
</verb></tscreen>

If you want to alter the value of a Spin Value relative to its current
value, then the following function can be used:

<tscreen><verb>
void Gtk_SpinButton::spin( GtkSpinType    direction,
                           gfloat         increment );
</verb></tscreen>

The <tt/direction/ parameter can take one of the following values:

<itemize>
<item> GTK_SPIN_STEP_FORWARD
<item> GTK_SPIN_STEP_BACKWARD
<item> GTK_SPIN_PAGE_FORWARD
<item> GTK_SPIN_PAGE_BACKWARD
<item> GTK_SPIN_HOME
<item> GTK_SPIN_END
<item> GTK_SPIN_USER_DEFINED
</itemize>

This function packs in quite a bit of functionality, which I will
attempt to clearly explain. Many of these settings use values from the
Adjustment object that is associated with a Spin Button.

GTK_SPIN_STEP_FORWARD and GTK_SPIN_STEP_BACKWARD change the value of
the Spin Button by the amount specified by <tt/increment/, unless
<tt/increment/ is equal to 0, in which case the value is changed by
the value of <tt/step_increment/ in theAdjustment.

GTK_SPIN_PAGE_FORWARD and GTK_SPIN_PAGE_BACKWARD simply alter the value of
the Spin Button by <tt/increment/.

GTK_SPIN_HOME sets the value of the Spin Button to the bottom of the
Adjustments range.

GTK_SPIN_END sets the value of the Spin Button to the top of the
Adjustments range.

GTK_SPIN_USER_DEFINED simply alters the value of the Spin Button by
the specified amount.

We move away from methods for setting and retreving the range attributes
of the Spin Button now, and move onto functions that effect the
appearance and behaviour of the Spin Button widget itself.

The first of these methods is used to constrain the text box of the
Spin Button such that it may only contain a numric value. This
prevents a user from typing anything other than numeric values into
the text box of a Spin Button:

<tscreen><verb>
void Gtk_SpinButton::set_numeric(bool       numeric);
</verb></tscreen>

You can set whether a Spin Button will wrap around between the upper
and lower range values with the following function:

<tscreen><verb>
void Gtk_SpinButton::set_wrap(bool       wrap);
</verb></tscreen>

You can set a Spin Button to round the value to the nearest
<tt/step_increment/, which is set within the Adjustment object used
with the Spin Button. This is accomplished with the following
method :

<tscreen><verb>
void Gtk_SpinButton::set_snap_to_ticks(bool        snap_to_ticks);
</verb></tscreen>

The update policy of a Spin Button can be changed with the following
method :

<tscreen><verb>
void Gtk_SpinButton::set_update_policy(GtkSpinButtonUpdatePolicy policy);
</verb></tscreen>

<!-- TODO: find out what this does - TRG -->

The possible values of <tt/policy/ are either GTK_UPDATE_ALWAYS or
GTK_UPDATE_IF_VALID.

These policies affect the behavior of a Spin Button when parsing
inserted text and syncing it's value with the values of the
Adjustment.

In the case of GTK_UPDATE_IF_VALID the Spin Button only value gets
changed if the text input is a numeric value that
is within the range specified by the Adjustment. Otherwise
the text is reset to the current value.

In case of GTK_UPDATE_ALWAYS we ignore errors while converting text
into a numeric value.

The appearance of the buttons used in a Spin Button can be changed
using the following function:

<tscreen><verb>
void Gtk_SpinButton::set_shadow_type(GtkShadowType  shadow_type );
</verb></tscreen>

As usual, the <tt/shadow_type/ can be one of:

<itemize>
<item> GTK_SHADOW_IN
<item> GTK_SHADOW_OUT
<item> GTK_SHADOW_ETCHED_IN
<item> GTK_SHADOW_ETCHED_OUT
</itemize>

Finally, you can explicitly request that a Spin Button update itself:

<tscreen><verb>
void Gtk_SpinButton::update();
</verb></tscreen>

It's example time again.

<tscreen><verb>
example_include(spinbutton/spinbutton.cc)
</verb></tscreen>

<!-- ----------------------------------------------------------------- -->
<sect1>Combo Box
<p>
The combo box is another fairly simple widget that is really just a
collection of other widgets. From the users point of view, the widget
consists of a text entry box and a pull down menu from which the user
can select one of a set of predefined entries. Alternatively, the user
can type a different option directly into the text box.

The Combo Box has two principle parts that you really care about: an
entry and a list. The ctor is 

<tscreen><verb>
Gtk_Combo();
</verb></tscreen>

Now, if you want to set the string in the entry section of the combo
box, this is done by obtaining the <tt/entry/ widget and manipulating
it :

<tscreen><verb>
    Gtk_Entry *comboEntry = combo.get_entry();
    comboEntry->set_text("My String.");
</verb></tscreen>

To set the values in the popdown list, one uses one of the following
methods:

<tscreen><verb>
void Gtk_Combo::set_popdown_strings(GList    *strings );
void Gtk_Combo::set_popdown_strings(const G_List_string &amp;strings);
</verb></tscreen>

The 1st one is kept to ease quick'n dirty translation of GTK+
applications to Gtk--. The 2nd one is preferred.

Before you can do this, you have to assemble a G_List_string of the
strings that you want. G_List_string is a specialization of G_List,
which is a wrapping of a linked list implementation that is part of
<ref id="sec_glib" name="glib">, a library supporing GTK. For the
moment, the quick and dirty explanation is that you need to declare a
G_List_string, then append strings to it with

<tscreen><verb>
G_List_string::append(char* string);
</verb></tscreen>

NOTE: Yes, char* and not string. Sorry. We're working on improving
this.

Here's a typical code segment for creating a set of options:

<tscreen><verb>
    G_List_string glist;

    glist.append("String 1");
    glist.append("String 2");
    glist.append("String 3"); 
    glist.append("String 4");

    combo.set_popdown_strings(glist) ;
</verb></tscreen>

At this point you have a working combo box that has been set up.
There are a few aspects of its behavior that you can change. These
are accomplished with the functions: 

<tscreen><verb>
void Gtk_Combo::set_use_arrows(bool      val);

void Gtk_Combo::set_use_arrows_always(bool      val);

void Gtk_Combo::set_case_sensitive(gint      val);
</verb></tscreen>

<tt/Gtk_Combo::set_use_arrows()/ lets the user change the value in the
entry using the up/down arrow keys. This doesn't bring up the list, but
rather replaces the current text in the entry with the next list entry
(up or down, as your key choice indicates). It does this by searching
in the list for the item corresponding to the current value in the
entry and selecting the previous/next item accordingly. Usually in an
entry the arrow keys are used to change focus (you can do that anyway
using TAB). Note that when the current item is the last of the list
and you press arrow-down it changes the focus (the same applies with
the first item and arrow-up).

If the current value in the entry is not in the list, then the
function of <tt/Gtk_Combo::set_use_arrows()/ is disabled.

<tt/Gtk_Combo::set_use_arrows_always()/ similarly allows the use the
the up/down arrow keys to cycle through the choices in the dropdown
list, except that it wraps around the values in the list, completely
disabling the use of the up and down arrow keys for changing focus.

<tt/Gtk_Combo::set_case_sensitive()/ toggles whether or not Gtk--
searches for entries in a case sensitive manner. This is used when
the Combo widget is asked to find a value from the list using the
current entry in the text box. This completion can be performed in
either a case sensitive or insensitive manner, depending upon the use
of this function. The Combo widget can also simply complete the
current entry if the user presses the key combination MOD-1 and
'Tab'. MOD-1 is often mapped to the 'Alt' key. Note, however that some
Window managers also use this key combination, which will override
it's use within Gtk--.

Now that we have a combo box, tailored to look and act how we want it,
all that remains is being able to get data from the combo box. This is
relatively straight forward. The majority of the time, all you are
going to care about getting data from is the entry. The entry is
accessed simply through the Gtk_Combo::get_entry() method. The two
principle things that you are going to want to do with it are attach
to the activate signal, which indicates that the user has pressed the
Return or Enter key,and read the text. The first is accomplished
using something like:

<tscreen><verb>
!!!!!!!!!!
    gtk_signal_connect(GTK_OBJECT(GTK_COMB(combo)->entry), "activate",
                       GTK_SIGNAL_FUNC (my_callback_function), my_data);
</verb></tscreen>

Getting the text at any arbitrary time is accomplished by simply using
the entry function:

<tscreen><verb>
string Gtk_Entry::get_text();
</verb></tscreen>

Such as:

<tscreen><verb>
    string s;
    Gtk_Entry *comboEntry = combo.get_entry();
    s = comboEntry->get_text();
</verb></tscreen>

That's about all there is to it. There is a function

<tscreen><verb>
void Gtk_Combo::disable_activate();
</verb></tscreen>

that will disable the activate signal on the entry widget in the combo
box. Personally, I can't think of why you'd want to use it, but it
does exist.

<!-- There are also a function to set the string on a particular item, void
gtk_combo_set_item_string(GtkCombo *combo, GtkItem *item, const gchar
*item_value), but this requires that you have a pointer to the
appropriate GtkItem.  Frankly, I have no idea how to do that.
-->

<!-- ----------------------------------------------------------------- -->
<sect1> Color Selection
<p>
The color selection widget is, not surprisingly, a widget for
interactive selection of colors. This composite widget lets the user
select a color by manipulating RGB (Red, Green, Blue) and HSV (Hue,
Saturation, Value) triples.  This is done either by adjusting single
values with sliders or entries, or by picking the desired color from a
hue-saturation wheel/value bar.  Optionally, the opacity of the color
can also be set.

The color selection widget currently emits only one signal,
"color_changed", which is emitted whenever the current color in the
widget changes, either when the user changes it or if it's set
explicitly through gtk_color_selection_set_color().

Lets have a look at what the color selection widget has to offer
us. The widget comes in two flavours: Gtk_ColorSelection and
Gtk_ColorSelectionDialog. The Gtk_ColorSelection has the usual "no
frills" ctor :

<tscreen><verb>
Gtk_ColorSelection();
</verb></tscreen>
	
You'll probably not be using this constructor directly. It creates an
orphan Gtk_ColorSelection widget which you'll have to parent
yourself. The Gtk_ColorSelection widget inherits from the Gtk_VBox
widget.

<tscreen><verb> 
Gtk_ColorSelectionDialog(const _gtk_string &amp;title);
</verb></tscreen>

This ctor creates a Gtk_ColorSelectionDialog, which inherits from a
Gtk_Dialog. It consists of a Gtk_Frame containing a Gtk_ColorSelection
widget, a Gtk_HSeparator and a Gtk_HBox with three buttons, "Ok",
"Cancel" and "Help". You can reach these buttons by using the
get_ok_button(), get_cancel_button() and get_help_button() methods.

<tscreen><verb>
void Gtk_ColorSelection::set_update_policy(GtkUpdateType      policy);
</verb></tscreen>

This function sets the update policy. The default policy is
GTK_UPDATE_CONTINUOUS which means that the current color is updated
continuously when the user drags the sliders or presses the mouse and
drags in the hue-saturation wheel or value bar. If you experience
performance problems, you may want to set the policy to
GTK_UPDATE_DISCONTINUOUS or GTK_UPDATE_DELAYED.

<tscreen><verb>
void Gtk_ColorSelection::set_opacity(gint               use_opacity );
</verb></tscreen>

The color selection widget supports adjusting the opacity of a color
(also known as the alpha channel). This is disabled by
default. Calling this function with use_opacity set to TRUE enables
opacity. Likewise, use_opacity set to FALSE will disable opacity.

<tscreen><verb>
void Gtk_ColorSelection::set_color(gdouble           *color );
</verb></tscreen>

You can set the current color explicitly by calling this function with
a pointer to an array of colors (gdouble). The length of the array
depends on whether opacity is enabled or not. Position 0 contains the
red component, 1 is green, 2 is blue and opacity is at position 3
(only if opacity is enabled, see
Gtk_ColorSelection::set_opacity()). All values are between 0.0 and
1.0.

<tscreen><verb>
void Gtk_ColorSelection::get_color(gdouble           *color );
</verb></tscreen>

When you need to query the current color, typically when you've
received a "color_changed" signal, you use this function. Color is a
pointer to the array of colors to fill in. See the
Gtk_ColorSelection::set_color() function for the description of this
array.

<!-- Need to do a whole section on DnD - TRG
Drag and drop
-------------

The color sample areas (right under the hue-saturation wheel) supports
drag and drop. The type of drag and drop is "application/x-color". The
message data consists of an array of 4 (or 5 if opacity is enabled)
gdouble values, where the value at position 0 is 0.0 (opacity on) or
1.0 (opacity off) followed by the red, green and blue values at
positions 1,2 and 3 respectively.  If opacity is enabled, the opacity
is passed in the value at position 4.
-->

Here's a simple example demonstrating the use of the
GtkColorSelectionDialog. The program displays a window containing a
drawing area. Clicking on it opens a color selection dialog, and
changing the color in the color selection dialog changes the
background color.

<tscreen><verb>
example_include(colorsel/colorsel.cc)
</verb></tscreen>

<!-- ----------------------------------------------------------------- -->
<sect1> File Selections
<p>
The file selection widget is a quick and simple way to display a File
dialog box. It comes complete with Ok, Cancel, and Help buttons, a
great way to cut down on programming time.

The file selection ctor is:

<tscreen><verb>
Gtk_FileSelection(const _gtk_string &amp;title);
</verb></tscreen>

To set the filename, for example to bring up a specific directory, or
give a default filename, use this function:

<tscreen><verb>
void Gtk_FileSelection::set_filename(const string &amp;filename);
</verb></tscreen>

To grab the text that the user has entered or clicked on, use this 
function:

<tscreen><verb>
string Gtk_FileSelection::get_filename();
</verb></tscreen>

There are also accessors to the widgets contained within the file 
selection widget. These are:

<itemize>
<item>get_selection_entry()
<item>get_selection_text()
<item>get_ok_button()
<item>get_cancel_button()
</itemize>

Most likely you will want to use the ok_button and cancel_button in
signaling their use.

Included here is an example stolen from testgtk--.cc, modified to run on
its own. As you will see, there is nothing much to creating a file
selection widget. While in this example the Help button appears on the
screen, it does nothing as there is not a signal attached to it.

<tscreen><verb>
example_include(filesel/filesel.cc)
</verb></tscreen>

<!-- ***************************************************************** -->
