<!-- ***************************************************************** -->
<sect>Packing Widgets
<!-- ***************************************************************** -->
<p>
When creating an application, you'll want to put more than one widget
inside a window. Our first <em>helloworld</em> example only used one
widget so we could simply use a Gtk_Container::add call to "pack" the
widget into the window. But when you want to put more than one widget
into a window, how do you control where that widget is positioned?
This is where packing comes in.

<!-- ----------------------------------------------------------------- -->
<sect1>Theory of Packing Boxes
<p>
Most packing is done by creating boxes as in the example above. These
are invisible widget containers that we can pack our widgets into
which come in two forms, a horizontal box, and a vertical box. When
packing widgets into a horizontal box, the objects are inserted
horizontally from left to right or right to left depending on the call
used. In a vertical box, widgets are packed from top to bottom or vice
versa. You may use any combination of boxes inside or beside other
boxes to create the desired effect.

To create a new horizontal box, we use a Gtk_HBox, and for vertical
boxes, a Gtk_VBox.The Gtk_Box::pack_start() and Gtk_Box::pack_end()
methods are used to place objects inside of these containers. The
Gtk_Box::pack_start() method will start at the top and work its way
down in a vbox, and pack left to right in an hbox.  Gtk_Box::pack_end()
will do the opposite, packing from bottom to top in a vbox, and right
to left in an hbox. Using these method allow us to right justify or
left justify our widgets and may be mixed in any way to achieve the
desired effect. We will use Gtk_Box::pack_start() in most of our
examples. An object may be another container or a widget. In fact,
many widgets are actually containers themselves, including the button,
but we usually only use a label inside a button.

By using these calls, Gtk-- knows where you want to place your widgets
so it can do automatic resizing and other nifty things. There's also a
number of options as to how your widgets should be packed. As you can
imagine, this method gives us a quite a bit of flexibility when
placing and creating widgets.

<!-- ----------------------------------------------------------------- -->
<sect1>Details of Boxes
<p>
Because of this flexibility, packing boxes in Gtk-- can be confusing at
first. There are a lot of options, and it's not immediately obvious how
they all fit together. In the end however, there are basically five
different styles.

<? <CENTER> >
<?
<IMG SRC="gtk_tut_packbox1.gif" VSPACE="15" HSPACE="10" WIDTH="528"
HEIGHT="235" ALT="Box Packing Example Image">
>
<? </CENTER> >

Each line contains one horizontal box (hbox) with several
buttons. Each of the buttons is packed into the hbox the same way
(i.e. same arguments to the Gtk_Box::pack_start() method).

This is the declaration of the Gtk_Box::pack_start method.

<tscreen><verb>
void Gtk_Box::pack_start(const Gtk_Widget &amp;child,
                         bool       expand,
                         bool       fill,
                         gint       padding );
</verb></tscreen>

The first argument is the object you're packing. The objects will all
be buttons for now, so we'll be packing buttons into boxes.

The expand argument to Gtk_Box::pack_start() and Gtk_Box::pack_end()
controls whether the widgets are laid out in the box to fill in all
the extra space in the box so the box is expanded to fill the area
alloted to it (TRUE).  Or the box is shrunk to just fit the widgets
(FALSE). Setting expand to FALSE will allow you to do right and left
justification of your widgets.  Otherwise, they will all expand to fit
into the box, and the same effect could be achieved by using only one
of Gtk_Box::pack_start or Gtk_Box::pack_end.

The fill argument to the Gtk_Box::pack_xxx methods control whether the
extra space is allocated to the objects themselves (TRUE), or as extra
padding in the box around these objects (FALSE). It only has an effect
if the expand argument is also TRUE.

When creating a new box, the ctor looks like this:

<tscreen><verb>
Gtk_Box(bool homogeneous,
        gint spacing);
</verb></tscreen>

The homogeneous argument controls whether each object in the box has
the same size (i.e. the same width in an hbox, or the same height in a
vbox). If it is set, the expand argument to the Gtk_Box::pack routines
is always turned on.

What's the difference between spacing (set when the box is created)
and padding (set when elements are packed)? Spacing is added between
objects, and padding is added on either side of an object. The
following figure should make it clearer:

<? <CENTER> >
<?
<IMG ALIGN="center" SRC="gtk_tut_packbox2.gif" WIDTH="509"
HEIGHT="213" VSPACE="15" HSPACE="10"
ALT="Box Packing Example Image">
>
<? </CENTER> >

Here is the code used to create the above images. I've commented it
fairly heavily so hopefully you won't have any problems following
it. Compile it yourself and play with it.

<!-- ----------------------------------------------------------------- -->
<sect1>Packing Demonstration Program
<p>
<tscreen><verb>
example_include(packbox/packbox.cc)
</verb></tscreen>

<!-- ----------------------------------------------------------------- -->
<sect1>Packing Using Tables
<p>
Let's take a look at another way of packing - Tables. These can be
extremely useful in certain situations.

Using tables, we create a grid that we can place widgets in. The
widgets may take up as many spaces as we specify.

The first thing to look at of course, is the Gtk_Table ctor:

<tscreen><verb>
Gtk_Table( gint rows,
           gint columns,
           bool homogeneous );
</verb></tscreen>

The first argument is the number of rows to make in the table, while
the second, obviously, is the number of columns.

The homogeneous argument has to do with how the table's boxes are
sized. If homogeneous is TRUE, the table boxes are resized to the size
of the largest widget in the table. If homogeneous is FALSE, the size
of a table boxes is dictated by the tallest widget in its same row,
and the widest widget in its column.

The rows and columns are laid out from 0 to n, where n was the number
specified in the call to gtk_table_new. So, if you specify rows = 2
and columns = 2, the layout would look something like this:

<tscreen><verb>
 0          1          2
0+----------+----------+
 |          |          |
1+----------+----------+
 |          |          |
2+----------+----------+
</verb></tscreen>

Note that the coordinate system starts in the upper left hand corner.
To place a widget into a box, use the following function:

<tscreen><verb>
void Gtk_Table::attach(Gtk_Widget &amp;child,
                       guint      left_attach,
                       guint      right_attach,
                       guint      top_attach,
                       guint      bottom_attach,
                       guint      xoptions,
                       guint      yoptions,
                       guint      xpadding,
                       guint      ypadding );
</verb></tscreen>				       

Where the first argument ("child") is the widget you wish to place in
the table.

The left and right attach arguments specify where to place the widget,
and how many boxes to use. If you want a button in the lower right
table entry of our 2x2 table, and want it to fill that entry ONLY,
left_attach would be = 1, right_attach = 2, top_attach = 1,
bottom_attach = 2.

Now, if you wanted a widget to take up the whole top row of our 2x2
table, you'd use left_attach = 0, right_attach = 2, top_attach = 0,
bottom_attach = 1.

The xoptions and yoptions are used to specify packing options and may
be bitwise OR'ed together to allow multiple options.

These options are:
<itemize>
<item>GTK_FILL - If the table box is larger than the widget, and
GTK_FILL is specified, the widget will expand to use all the room
available.

<item>GTK_SHRINK - If the table widget was allocated less space then
was requested (usually by the user resizing the window), then the
widgets would normally just be pushed off the bottom of the window and
disappear. If GTK_SHRINK is specified, the widgets will shrink with
the table.

<item>GTK_EXPAND - This will cause the table to expand to use up any
remaining space in the window.
</itemize>

Padding is just like in boxes, creating a clear area around the widget
specified in pixels.

gtk_table_attach() has a LOT of options, but some of them are assigned 
reasonable default values:

<tscreen><verb>
void Gtk_Table::attach(Gtk_Widget &amp;child,
	               guint left_attach,
	               guint right_attach,
	               guint top_attach,
	               guint bottom_attach,
	               gint xoptions=(GTK_FILL|GTK_EXPAND), 
	               gint yoptions=(GTK_FILL|GTK_EXPAND),
	               gint xpadding=0, gint ypadding=0);
</verb></tscreen>

The X and Y options default to GTK_FILL | GTK_EXPAND, and X and Y
padding default to 0.

We also have Gtk_Table::set_row_spacing() and
Gtk_Table::set_col_spacing().  This places spacing between the rows at
the specified row or column.

<tscreen><verb>
void Gtk_Table::set_row_spacing(gint      row,
                                gint      spacing );
</verb></tscreen>

and

<tscreen><verb>
void Gtk_Table::set_col_spacing (gint      column,
                                 gint      spacing );
</verb></tscreen>

Note that for columns, the space goes to the right of the column, and
for rows, the space goes below the row.

You can also set a consistent spacing of all rows and/or columns with:

<tscreen><verb>
void Gtk_Table::set_row_spacings(gint      spacing );
</verb></tscreen>

And,

<tscreen><verb>
void Gtk_Table::set_col_spacings(gint      spacing );
</verb></tscreen>

Note that with these calls, the last row and last column do not get
any spacing.

<!-- ----------------------------------------------------------------- -->
<sect1>Table Packing Example
<p>
Here we make a window with three buttons in a 2x2 table.
The first two buttons will be placed in the upper row.
A third, quit button, is placed in the lower row, spanning both columns.
Which means it should look something like this:

<? <CENTER> >
<?
<IMG SRC="gtk_tut_table.gif" VSPACE="15" HSPACE="10" 
ALT="Table Packing Example Image" WIDTH="180" HEIGHT="120">
>
<? </CENTER> >

Here's the source code:

<tscreen><verb>
example_include(table/table.cc)
</verb></tscreen>

