<sect> Text Widget
<!-- ***************************************************************** -->
<p>
The Text widget allows multiple lines of text to be displayed and
edited.  It supports both multi-colored and multi-font text, allowing
them to be mixed in any way we wish. It also has a wide set of key
based text editing commands, which are compatible with Emacs.

The text widget supports full cut-and-paste facilities, including the
use of double- and triple-click to select a word and a whole line,
respectively.

<!-- ----------------------------------------------------------------- -->
<sect1>Creating and Configuring a Text box
<p>
There is only one function for creating a new Text widget.

<tscreen><verb>
GtkWidget *gtk_text_new( GtkAdjustment *hadj,
                         GtkAdjustment *vadj );
</verb></tscreen>

The arguments allow us to give the Text widget pointers to Adjustments
that can be used to track the viewing position of the widget. Passing
NULL values to either or both of these arguments will cause the
gtk_text_new function to create its own.

<tscreen><verb>
void gtk_text_set_adjustments( GtkText       *text,
                               GtkAdjustment *hadj,
                               GtkAdjustment *vadj );
</verb></tscreen>

The above function allows the horizontal and vertical adjustments of a
text widget to be changed at any time.

The text widget will not automatically create its own scrollbars when
the amount of text to be displayed is too long for the display
window. We therefore have to create and add them to the display layout
ourselves.

<tscreen><verb>
  vscrollbar = gtk_vscrollbar_new (GTK_TEXT(text)->vadj);
  gtk_box_pack_start(GTK_BOX(hbox), vscrollbar, FALSE, FALSE, 0);
  gtk_widget_show (vscrollbar);
</verb></tscreen>

The above code snippet creates a new vertical scrollbar, and attaches
it to the vertical adjustment of the text widget, <tt/text/. It then
packs it into a box in the normal way.

Note, currently the GtkText widget does not support horizontal
scrollbars.

There are two main ways in which a Text widget can be used: to allow
the user to edit a body of text, or to allow us to display multiple
lines of text to the user. In order for us to switch between these
modes of operation, the text widget has the following function:

<tscreen><verb>
void gtk_text_set_editable( GtkText *text,
                            gint     editable );
</verb></tscreen>

The <tt/editable/ argument is a TRUE or FALSE value that specifies
whether the user is permitted to edit the contents of the Text
widget. When the text widget is editable, it will display a cursor at
the current insertion point.

You are not, however, restricted to just using the text widget in
these two modes. You can toggle the editable state of the text widget
at any time, and can insert text at any time.

The text widget wraps lines of text that are too long to fit onto a
single line of the display window. Its default behaviour is to break
words across line breaks. This can be changed using the next function:

<tscreen><verb>
void gtk_text_set_word_wrap( GtkText *text,
                             gint     word_wrap );
</verb></tscreen>

Using this function allows us to specify that the text widget should
wrap long lines on word boundaries. The <tt/word_wrap/ argument is a
TRUE or FALSE value.

<!-- ----------------------------------------------------------------- -->
<sect1>Text Manipulation
<P>
The current insertion point of a Text widget can be set using
<tscreen><verb>
void gtk_text_set_point( GtkText *text,
                         guint    index );
</verb></tscreen>

where <tt/index/ is the position to set the insertion point.

Analogous to this is the function for getting the current insertion
point:

<tscreen><verb>
guint gtk_text_get_point( GtkText *text );
</verb></tscreen>

A function that is useful in combination with the above two functions
is

<tscreen><verb>
guint gtk_text_get_length( GtkText *text );
</verb></tscreen>

which returns the current length of the Text widget. The length is the
number of characters that are within the text block of the widget,
including characters such as carriage-return, which marks the end of
lines.

In order to insert text at the current insertion point of a Text
widget, the function gtk_text_insert is used, which also allows us to
specify background and foreground colors and a font for the text.

<tscreen><verb>
void gtk_text_insert( GtkText    *text,
                      GdkFont    *font,
                      GdkColor   *fore,
                      GdkColor   *back,
                      const char *chars,
                      gint        length );
</verb></tscreen>

Passing a value of <tt/NULL/ in as the value for the foreground color,
background colour or font will result in the values set within the
widget style to be used. Using a value of <tt/-1/ for the length
parameter will result in the whole of the text string given being
inserted.

The text widget is one of the few within Gtk-- that redraws itself
dynamically, outside of the gtk_main function. This means that all
changes to the contents of the text widget take effect
immediately. This may be undesirable when performing multiple changes
to the text widget. In order to allow us to perform multiple updates
to the text widget without it continuously redrawing, we can freeze
the widget, which temporarily stops it from automatically redrawing
itself every time it is changed. We can then thaw the widget after our
updates are complete.

The following two functions perform this freeze and thaw action:

<tscreen><verb>
void gtk_text_freeze( GtkText *text );

void gtk_text_thaw( GtkText *text );         
</verb></tscreen>

Text is deleted from the text widget relative to the current insertion
point by the following two functions. The return value is a TRUE or
FALSE indicator of whether the operation was successful.

<tscreen><verb>
gint gtk_text_backward_delete( GtkText *text,
                               guint    nchars );

gint gtk_text_forward_delete ( GtkText *text,
                               guint    nchars );
</verb></tscreen>

If you want to retrieve the contents of the text widget, then the
macro <tt/GTK_TEXT_INDEX(t, index)/ allows you to retrieve the
character at position <tt/index/ within the text widget <tt/t/.

To retrieve larger blocks of text, we can use the function

<tscreen><verb>
gchar *gtk_editable_get_chars( GtkEditable *editable,
                               gint         start_pos,
                               gint         end_pos );   
</verb></tscreen>

This is a function of the parent class of the text widget. A value of
-1 as <tt/end_pos/ signifies the end of the text. The index of the
text starts at 0.

The function allocates a new chunk of memory for the text block, so
don't forget to free it with a call to g_free when you have finished
with it.
 
<!-- ----------------------------------------------------------------- -->
<sect1>Keyboard Shortcuts
<p>
The text widget has a number of pre-installed keyboard shortcuts for
common editing, motion and selection functions. These are accessed
using Control and Alt key combinations.

In addition to these, holding down the Control key whilst using cursor
key movement will move the cursor by words rather than
characters. Holding down Shift whilst using cursor movement will
extend the selection.

<sect2>Motion Shortcuts
<p>
<itemize>
<item> Ctrl-A   Beginning of line
<item> Ctrl-E   End of line
<item> Ctrl-N   Next Line
<item> Ctrl-P   Previous Line
<item> Ctrl-B   Backward one character
<item> Ctrl-F   Forward one character
<item> Alt-B    Backward one word
<item> Alt-F    Forward one word
</itemize>

<sect2>Editing Shortcuts
<p>
<itemize>
<item> Ctrl-H   Delete Backward Character (Backspace)
<item> Ctrl-D   Delete Forward Character (Delete)
<item> Ctrl-W   Delete Backward Word
<item> Alt-D    Delete Forward Word
<item> Ctrl-K   Delete to end of line
<item> Ctrl-U   Delete line
</itemize>

<sect2>Selection Shortcuts
<p>
<itemize>
<item> Ctrl-X   Cut to clipboard
<item> Ctrl-C   Copy to clipboard
<item> Ctrl-V   Paste from clipboard
</itemize>

<!-- ----------------------------------------------------------------- -->
<sect1>A GtkText Example
<p>
<tscreen><verb>
example_include(text/text.cc)
</verb></tscreen>


<!-- ***************************************************************** -->
