<sect>Timeouts, IO and Idle Functions<label id="sec_timeouts">
<!-- ***************************************************************** -->

<!-- ----------------------------------------------------------------- -->
<sect1>Timeouts
<p>
You may be wondering how to make Gtk-- do useful work when in gtk_main.
Well, you have several options. Using the following functions you can
create a timeout function that will be called every "interval"
milliseconds.

<tscreen><verb>
Connection Gtk_Main::timeout.connect(const SlotType &sd, guint32 interval);
</verb></tscreen>

The first argument is a slot you wish to have called when the timeout
occurs. The second argument is the number of milliseconds between
calls to your function. You get back a Connection object that can be
used to destroy the connection. <em>NOTE:</em> When the Connection
object is deleted the connection still exists. You have to call

<tscreen><verb>
MyConnection.disconnect();
</verb></tscreen>

to testroy the connection. An other way of destroying the Connection
is your callback function. It has to be of the type
<tt>Slot0&lt;gint&gt;</tt>. As you see from the definition your
callback function has to return a value of the type <tt>gint</tt>.  A
definition of a sample function might look like this:

<tscreen><verb>
gint MyCallback() { cout << "Hello World!"; return true; }
</verb></tscreen>

You may stop the timeout function by returning zero or 'false' from
your callback function. Obviously this means that if you want your
function to continue to be called, it should return a non-zero value,
i.e. 'true'.

For your remembrance a few ways to create a slot from a callback function
(In C++ I think the slot from a member function is most important).

<tscreen><verb>
gint timeout_callback( gpointer data );
</verb></tscreen>


You can create a slot for your callback function like this:
<tscreen><verb>
class MyClass { public: gint ClassMember(); }  mc;
gint MyFunction();
gint MyGintFunction(gint);

Slot0&lt;gint&gt; MySlot = slot(MyFunction);
Slot0&lt;gint&gt; MyGintSlot = bind(slot(MyGintFunction),123);
Slot0&lt;gint&gt; MyMemberSlot = slot(mc, &MyClass::ClassMember);
</verb></tscreen>

At this point a little example might be nice. Here you are:
<tscreen><verb>
example_include(timeout/timeout.cc)
</verb></tscreen>


<!-- ----------------------------------------------------------------- -->
<sect1>Monitoring IO
<p>
A nifty feature of GDK (one of the library that underlies Gtk--), is the
ability to have it check for data on a file descriptor for you (as
returned by open(2) or socket(2)). This is especially useful for
networking applications. The function:

<tscreen><verb>
Connection Gtk_Main::input.connect(const SlotType& sd, gint source,
                                   GdkInputCondition condition);
</verb></tscreen>

The first argument is a slot you wish to have called when then the specified
event (see argument three) occurs on the filedescriptor you specify
using argument two. Argument three may be one or a combination(<tt>||</tt>) of:

<itemize>

<item>GDK_INPUT_READ - Call your function when there is data ready for
reading on your file descriptor.

<item>GDK_INPUT_WRITE - Call your function when the file descriptor is
ready for writing.

<item>GDK_INPUT_EXCEPTION - Call your function when an exception happened
on the filedescriptor.
</itemize>

The return value is a Connection that may be used to stop monitoring
this file descriptor using the <tt>disconnect</tt> following function.
The callback function should be declared as:

<tscreen><verb>
void input_callback(gint source, GdkInputCondition condition );
</verb></tscreen>

Where <tt/source/ and <tt/condition/ are as specified above. As usual
the slot is created with <tt>slot(...)</tt> and can be a member function
of an object.

<p>
A little (and somewhat dirty) example follows as usual.
<tscreen><verb>
example_include(input/input.cc)
</verb></tscreen>
To use the example (that does not even have a window)
just execute it from a terminal. It will create a pipe
named <tt>testpipe</tt> in the current directory. Then start
an other shell and execute <tt>cat >testpipe</tt>. The example
will print each line you enter until you type <tt>quit</tt>.

<!-- ----------------------------------------------------------------- -->
<sect1>Idle Functions
<p>

What if you have a function you want called when nothing else is
happening ?

<tscreen><verb>
Connection Gtk_Main::idle.connect(Slot0<gint> idlefunc, gint priority);
</verb></tscreen>

This causes Gtk-- to call the specified function whenever nothing else
is happening. You can add a priority (lower numbers are higher priorities).
If You dont use any value here GTK_PRIORITY_DEFAULT will be used. There are
two ways to remove the callback: calling <tt>disconnect</tt> on the Connection
object or returning false or 0 in the callback function which may be declared as:

<tscreen><verb>
int idleFunc();
</verb></tscreen>

Since this is very similar to the functions above this exlanations should
be sufficient to understand what's going on. However, there's a little example:

<tscreen><verb>
example_include(idle/idle.cc)
</verb></tscreen>

This example points out the diffrence of idle and timeout functions a
little.  If you need functions that are called periodically and speed
is not very important then you are lucky wich timeout functions. If
you want functions that are called as often as possible (like
calculating a fractal in background) then use idle functions.

Try executing the example and increasing the system load. The upper Progressbar
is increasing steadily while the lower one is dependend from system load.

<!-- ***************************************************************** -->
