<!-- ***************************************************************** -->
<sect>Widget Overview
<!-- ***************************************************************** -->
<p>
The general steps to creating a widget in Gtk-- are:
<enum>
<item> Declare a variable of the type of the widget you wish to use
These are all detailed in this section.

<item> Connect all signals and events you wish to use to the
appropriate handlers.

<item> Set the attributes of the widget.

<item> Pack the widget into a container using the appropriate call
such as Gtk_Container::add() or Gtk_Box::pack_start().

<item> Gtk_Widget::show() the widget.
</enum>

Gtk_Widget::show() lets Gtk-- know that we are done setting the
attributes of the widget, and it is ready to be displayed. You may
also use Gtk_Widget::hide to make it disappear again. The order in
which you show the widgets is not important, but I suggest showing the
window last so the whole window pops up at once rather than seeing the
individual widgets come up on the screen as they're formed. The
children of a widget (a window is a widget too) will not be displayed
until the window itself is shown using the Gtk_Widget::show() function.

<!-- ----------------------------------------------------------------- -->
<sect1>Widget Hierarchy
<p>
For your reference, here is the class hierarchy tree used to implement widgets.

<tscreen><verb>
 Gtk_Object
  +Gtk_Widget
  | +Gtk_Misc
  | | +Gtk_Label
  | | | +Gtk_AccelLabel
  | | | `Gtk_TipsQuery
  | | +Gtk_Arrow
  | | +Gtk_Image
  | | `Gtk_Pixmap
  | +Gtk_Container
  | | +Gtk_Bin
  | | | +Gtk_Alignment
  | | | +Gtk_Frame
  | | | | `Gtk_AspectFrame
  | | | +Gtk_Button
  | | | | +Gtk_ToggleButton
  | | | | | `Gtk_CheckButton
  | | | | |   `Gtk_RadioButton
  | | | | `Gtk_OptionMenu
  | | | +Gtk_Item
  | | | | +Gtk_MenuItem
  | | | | | +Gtk_CheckMenuItem
  | | | | | | `Gtk_RadioMenuItem
  | | | | | `Gtk_TearoffMenuItem
  | | | | +Gtk_ListItem
  | | | | `Gtk_TreeItem
  | | | +Gtk_Window
  | | | | +Gtk_ColorSelectionDialog
  | | | | +Gtk_Dialog
  | | | | | `Gtk_InputDialog
  | | | | +Gtk_DrawWindow
  | | | | +Gtk_FileSelection
  | | | | +Gtk_FontSelectionDialog
  | | | | `Gtk_Plug
  | | | +Gtk_EventBox
  | | | +Gtk_HandleBox
  | | | +Gtk_ScrolledWindow
  | | | `Gtk_Viewport
  | | +Gtk_Box
  | | | +Gtk_ButtonBox
  | | | | +Gtk_HButtonBox
  | | | | `Gtk_VButtonBox
  | | | +Gtk_VBox
  | | | | +Gtk_ColorSelection
  | | | | `Gtk_GammaCurve
  | | | `Gtk_HBox
  | | |   +Gtk_Combo
  | | |   `Gtk_Statusbar
  | | +Gtk_CList
  | | | `Gtk_CTree
  | | +Gtk_Fixed
  | | +Gtk_Notebook
  | | | `Gtk_FontSelection
  | | +Gtk_Paned
  | | | +Gtk_HPaned
  | | | `Gtk_VPaned
  | | +Gtk_Layout
  | | +Gtk_List
  | | +Gtk_MenuShell
  | | | +Gtk_MenuBar
  | | | `Gtk_Menu
  | | +Gtk_Packer
  | | +Gtk_Socket
  | | +Gtk_Table
  | | +Gtk_Toolbar
  | | `Gtk_Tree
  | +Gtk_Calendar
  | +Gtk_DrawingArea
  | | `Gtk_Curve
  | +Gtk_Editable
  | | +Gtk_Entry
  | | | `Gtk_SpinButton
  | | `Gtk_Text
  | +Gtk_Ruler
  | | +Gtk_HRuler
  | | `Gtk_VRuler
  | +Gtk_Range
  | | +Gtk_Scale
  | | | +Gtk_HScale
  | | | `Gtk_VScale
  | | `Gtk_Scrollbar
  | |   +Gtk_HScrollbar
  | |   `Gtk_VScrollbar
  | +Gtk_Separator
  | | +Gtk_HSeparator
  | | `Gtk_VSeparator
  | +Gtk_Preview
  | `Gtk_Progress
  |   `Gtk_ProgressBar
  +Gtk_Data
  | +Gtk_Adjustment
  | `Gtk_Tooltips
  `Gtk_ItemFactory
</verb></tscreen>

<!-- ----------------------------------------------------------------- -->
<sect1>Widgets Without Windows
<p>
The following widgets do not have an associated window. If you want to
capture events, you'll have to use the Gtk_EventBox. See the section on
the <ref id="sec_EventBox" name="EventBox"> widget.

<tscreen><verb>
Gtk_Alignment
Gtk_Arrow
Gtk_Bin
Gtk_Box
Gtk_Image
Gtk_Item
Gtk_Label
Gtk_Pixmap
Gtk_ScrolledWindow
Gtk_Separator
Gtk_Table
Gtk_AspectFrame
Gtk_Frame
Gtk_VBox
Gtk_HBox
Gtk_VSeparator
Gtk_HSeparator
</verb></tscreen>

We'll further our exploration of Gtk-- by examining each widget in
turn, creating a few simple functions to display them. Another good
source are the various modules in the testgtk--.cc program that comes
with Gtk--. They can be found in gtk--/examples/testgtk--/.

