/* example-start buttonbox buttonbox.c */

#include <gtk--/box.h>
#include <gtk--/buttonbox.h>
#include <gtk--/frame.h>
#include <gtk--/button.h>
#include <gtk--/window.h>
#include <gtk--/main.h>

class BBox : public Gtk_Frame
{
public:
  BBox                 (gint horizontal,
                        const string& title,
                        gint  spacing,
                        gint  child_w,
                        gint  child_h,
                        GtkButtonBoxStyle  layout);
};

BBox::BBox             (gint horizontal,
                        const string& title,
                        gint  spacing,
                        gint  child_w,
                        gint  child_h,
                        GtkButtonBoxStyle  layout)
  : Gtk_Frame(title)
{
  Gtk_ButtonBox *bbox;
  Gtk_Button *button;

  if (horizontal)
    bbox = manage( new Gtk_HButtonBox () );
  else
    bbox = manage( new Gtk_VButtonBox () );

  bbox->set_border_width (5);

  add (*bbox);

  /* Set the appearance of the Button Box */
  bbox->set_layout (layout);
  bbox->set_spacing (spacing);
  bbox->set_child_size (child_w, child_h);

  button = manage( new Gtk_Button ("OK"));
  bbox->add (*button);

  button = manage( new Gtk_Button ("Cancel"));
  bbox->add (*button);

  button = manage( new Gtk_Button ("Help"));
  bbox->add (*button);

}

class AppWindow : public Gtk_Window
{
public:
  AppWindow ();
  ~AppWindow ();

  virtual gint delete_event_impl (GdkEventAny*);
};

AppWindow::AppWindow()
  : Gtk_Window(GTK_WINDOW_TOPLEVEL)
{
  Gtk_Box *main_vbox;
  Gtk_Box *vbox;
  Gtk_Box *hbox;
  Gtk_Frame *frame_horz;
  Gtk_Frame *frame_vert;

  set_title ("Button Boxes");
  set_border_width (10);

  main_vbox = manage( new Gtk_VBox(FALSE, 0) );
  add (*main_vbox);

  frame_horz = manage( new Gtk_Frame ("Horizontal Button Boxes") );
  main_vbox->pack_start (*frame_horz, TRUE, TRUE, 10);

  vbox = manage( new Gtk_VBox (FALSE, 0) );
  vbox -> set_border_width (10);
  frame_horz-> add (*vbox);

  vbox->pack_start (*manage( new BBox (TRUE, "Spread (spacing 40)", 
                                      40, 85, 20, GTK_BUTTONBOX_SPREAD)),
		      TRUE, TRUE, 0);

  vbox->pack_start (*manage( new BBox (TRUE, "Edge (spacing 30)", 
	                              30, 85, 20, GTK_BUTTONBOX_EDGE)),
		      TRUE, TRUE, 5);

  vbox->pack_start (*manage( new BBox (TRUE, "Start (spacing 20)", 
                                      20, 85, 20, GTK_BUTTONBOX_START)),
		      TRUE, TRUE, 5);

  vbox->pack_start (*manage( new BBox (TRUE, "End (spacing 10)", 
                                      10, 85, 20, GTK_BUTTONBOX_END)),
		      TRUE, TRUE, 5);

  frame_vert = manage( new Gtk_Frame ("Vertical Button Boxes") );
  main_vbox->pack_start (*frame_vert, TRUE, TRUE, 10);

  hbox = manage( new Gtk_HBox (FALSE, 0) );
  hbox->set_border_width (10);
  frame_vert->add (*hbox);

  hbox->pack_start (*manage( new BBox (FALSE, "Spread (spacing 5)", 
                                      5, 85, 20, GTK_BUTTONBOX_SPREAD)),
		      TRUE, TRUE, 0);

  hbox->pack_start (*manage( new BBox (FALSE, "Edge (spacing 30)", 
	                              30, 85, 20, GTK_BUTTONBOX_EDGE)),
		      TRUE, TRUE, 5);

  hbox->pack_start (*manage( new BBox (FALSE, "Start (spacing 20)", 
                                      20, 85, 20, GTK_BUTTONBOX_START)),
		      TRUE, TRUE, 5);

  hbox->pack_start (*manage( new BBox (FALSE, "End (spacing 10)", 
                                      10, 85, 20, GTK_BUTTONBOX_END)),
		      TRUE, TRUE, 5);

  show_all ();

}

AppWindow::~AppWindow() {}

gint AppWindow::delete_event_impl (GdkEventAny*)
{
  Gtk_Main::quit();
  return 0;
}

int main( int   argc,
          char *argv[] )
{
  Gtk_Main main (argc,argv);
  AppWindow window;

  /* Enter the event loop */
  Gtk_Main::run ();
    
  return(0);
}
/* example-end */
