/* app.cc
 *
 * Copyright (C) 1999 Havoc Pennington, The Gtk-- Development Team
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

/*** gnomehello-app */
#include <cstdio>

#include <algorithm>

#include <gnome.h>
#include <libgnome/gnome-i18n.h>
#include <gnome--/main.h>
#include "application.h"

/* Keep a list of all open application windows */
static GSList* app_list = NULL;

void
Hello_App::nothing_cb()
{
  cerr << "nothing here\n";
}


void
Hello_App::install_menus_and_toolbar()
{

  using namespace Gnome_Menu_Helpers;
  
  // File menu
  //
  Gnome_UIInfoTree *fileMenuTree = new Gnome_UIInfoTree();
  
  fileMenuTree->push_back(MenuItems::New(N_("New Hello"),
					N_("Create a new hello"),
					slot(this, &Hello_App::nothing_cb)));
  fileMenuTree->push_back(MenuItems::Open(slot(this, &Hello_App::nothing_cb)));
  fileMenuTree->push_back(MenuItems::Save(slot(this, &Hello_App::nothing_cb)));
  fileMenuTree->push_back(MenuItems::SaveAs(slot(this, &Hello_App::nothing_cb)));

  fileMenuTree->push_back(Gnome_UISeparator());

  fileMenuTree->push_back(MenuItems::Close(slot(this, &Hello_App::nothing_cb)));
  fileMenuTree->push_back(MenuItems::Exit(slot(this, &Hello_App::nothing_cb)));

  Menus::File *fileMenu = new Menus::File(*fileMenuTree);

  // Edit menu
  //
  Gnome_UIInfoTree *editMenuTree = new Gnome_UIInfoTree();

  editMenuTree->push_back(MenuItems::Cut(slot(this, &Hello_App::nothing_cb)));
  editMenuTree->push_back(MenuItems::Copy(slot(this, &Hello_App::nothing_cb)));
  editMenuTree->push_back(MenuItems::Paste(slot(this, &Hello_App::nothing_cb)));
  editMenuTree->push_back(MenuItems::SelectAll(slot(this, &Hello_App::nothing_cb)));
  editMenuTree->push_back(MenuItems::Clear(slot(this, &Hello_App::nothing_cb)));
  editMenuTree->push_back(MenuItems::Undo(slot(this, &Hello_App::nothing_cb)));
  editMenuTree->push_back(MenuItems::Redo(slot(this, &Hello_App::nothing_cb)));
  editMenuTree->push_back(MenuItems::Find(slot(this, &Hello_App::nothing_cb)));
  editMenuTree->push_back(MenuItems::FindAgain(slot(this, &Hello_App::nothing_cb)));
  editMenuTree->push_back(MenuItems::Replace(slot(this, &Hello_App::nothing_cb)));
  editMenuTree->push_back(MenuItems::Properties(slot(this, &Hello_App::nothing_cb)));

  Menus::Edit *editMenu = new Menus::Edit(*editMenuTree);

  // Help menu
  //
  Gnome_UIInfoTree *helpMenuTree = new Gnome_UIInfoTree();

  helpMenuTree->push_back(Gnome_UIHelp("Gnome--hello"));
  helpMenuTree->push_back(MenuItems::About(slot(this, &Hello_App::nothing_cb)));

  Menus::Help *helpMenu = new Menus::Help(*helpMenuTree);

  Gnome_UIInfoTree *menus = new Gnome_UIInfoTree();

  menus->push_back(*fileMenu);
  menus->push_back(*editMenu);
  menus->push_back(*helpMenu);

  create_menus(*menus);

  // Toolbar
  //
  Gnome_UIInfoTree *toolbarTree = new Gnome_UIInfoTree();

  toolbarTree->push_back(Gnome_UIItemStock(N_("New Hello"),
					   N_("Create a new hello"),
					   slot(this, &Hello_App::nothing_cb),
					   GNOME_STOCK_PIXMAP_NEW));
  toolbarTree->push_back(Gnome_UISeparator());
  toolbarTree->push_back(Gnome_UIItemStock(N_("Prev"),
					   N_("Previous hello"),
					   slot(this, &Hello_App::nothing_cb),
					   GNOME_STOCK_PIXMAP_BACK));
  toolbarTree->push_back(Gnome_UIItemStock(N_("Next"),
					   N_("Next hello"),
					   slot(this, &Hello_App::nothing_cb),
					   GNOME_STOCK_PIXMAP_FORWARD));


  printf("status gtkobj 2 : %x\n", GTK_OBJECT_TYPE(status_.gtkobj()));
  create_toolbar(*toolbarTree);
  printf("status gtkobj 3 : %x\n", GTK_OBJECT_TYPE(status_.gtkobj()));

  install_menu_hints(*menus);
  printf("status gtkobj 4 : %x - type : %x\n",
	 status_.gtkobj(),
	 GTK_OBJECT_TYPE(status_.gtkobj()));
}



Hello_App::Hello_App(const string &message, 
		     const string &geometry,
		     GSList* greet)
  : Gnome_App("GnomeHello", "Gnome Hello"),
    status_(false, true, GNOME_PREFERENCES_NEVER)
{

  if(message.length())
    label_.set_text(message);
  else
    label_.set_text("Hello, World!");

  /*** gnomehello-widgets */

  set_policy(false, true, false);
  set_default_size(250, 350);
  set_wmclass("hello", "GnomeHello");

  frame_.set_shadow_type(GTK_SHADOW_IN);

  button_.set_border_width(10);

  button_.add(label_);
  frame_.add(button_);

  set_contents(frame_);

  set_statusbar(status_);

  install_menus_and_toolbar();

  /* gnomehello-widgets ***/
  
  /*** gnomehello-signals */
//   delete_event.connect(slot(this, &Hello_App::delete_event_cb));

//   button_.clicked.connect(bind(slot(this, &Hello_App::button_click_cb), (void*)&label_));
  /* gnomehello-signals ***/

  /*** gnomehello-geometry */
  if (geometry.length()) 
    {
      gint x, y, w, h;
      if ( gnome_parse_geometry( geometry.c_str(), 
                                 &x, &y, &w, &h ) ) 
        {
          if (x != -1)
            {
              set_uposition(x, y);
            }

          if (w != -1) 
            {
              set_default_size(w, h);
            }
        }
      else 
        {
          g_error(_("Could not parse geometry string `%s'"), geometry.c_str());
        }
    }

  /* gnomehello-geometry ***/

  if (greet != NULL)
    {
      gchar* greetings = g_strdup(_("Special Greetings to:\n"));
      GSList* tmp = greet;

      while (tmp != NULL)
        {
          gchar* old = greetings;

          greetings = g_strconcat(old, 
                                  (gchar*) tmp->data,
                                  "\n",
                                  NULL);
          g_free(old);

          tmp = g_slist_next(tmp);
        }
      
      Gnome_Dialog *dialog = Gnome_Dialog::ok_dialog(greetings);

      dialog->set_parent(*this);
    }

  app_list = g_slist_prepend(app_list, this);

  show_all();
  
}

void       
Hello_App::my_close()
{
  //app_list = g_slist_remove(app_list, app);

  destroy();

  if (app_list == NULL)
    {
      /* No windows remaining */
      Gtk_Main::quit();
    }
}

/*** gnomehello-quit */
gint 
Hello_App::delete_event_impl(GdkEventAny* e)
{
  my_close();

  /* Prevent the window's destruction, since we destroyed it 
   * ourselves with hello_app_close()
   */
  return true;
}
/* gnomehello-quit ***/

void 
Hello_App::button_click_cb(gpointer data)
{

  Gtk_Label *label = gtkmm_wrap(GTK_LABEL(data));

  string s(label->get());

  reverse(s.begin(), s.end());

  label->set(s);
}
