#include <gtk--/box.h>
#include <gtk--/frame.h>
#include <gtk--/label.h>
#include <gtk--/window.h>
#include <gtk--/main.h>

class AppWindow : Gtk_Window
{
public: 
  AppWindow();
  ~AppWindow();

  /* It's a good idea to do this for all application windows. */
  gint delete_event_impl (GdkEventAny*)
  {
    Gtk_Main::quit();
    return 0;
  }

};

AppWindow::AppWindow()
  : Gtk_Window (GTK_WINDOW_TOPLEVEL)
{
  Gtk_Box *hbox;
  Gtk_Box *vbox;
  Gtk_Frame* frame;
  Gtk_Label* label;

  /* Set some window properties */
  set_title("Label Example");
  set_border_width (5);

  /* Here we connect the "destroy" event to a signal handler */ 
  destroy.connect (Gtk_Main::quit.slot());

  vbox = manage( new Gtk_VBox (FALSE, 5) );
  hbox = manage( new Gtk_HBox (FALSE, 5) );
  add (*hbox);
  hbox->pack_start (*vbox, FALSE, FALSE);

  frame = manage( new Gtk_Frame ("Normal Label") );
  label = manage( new Gtk_Label ("This is a Normal label") );
  frame->add (*label);
  vbox->pack_start (*frame, FALSE, FALSE);
  
  frame = manage( new Gtk_Frame ("Multi-line Label") );
  label = manage( new Gtk_Label ("This is a Multi-line label.\nSecond line\n" \
			 "Third line"));
  frame->add (*label);
  vbox->pack_start (*frame, FALSE, FALSE);

  frame = manage( new Gtk_Frame ("Left Justified Label") );
  label = manage( new Gtk_Label ("This is a Left-Justified\n" \
			 "Multi-line label.\nThird      line"));
  label->set_justify (GTK_JUSTIFY_LEFT);
  frame->add (*label);
  vbox->pack_start (*frame, FALSE, FALSE);
  
  frame = manage( new Gtk_Frame ("Right Justified Label"));
  label = manage( new Gtk_Label ("This is a Right-Justified\nMulti-line label.\n" \
			 "Fourth line, (j/k)"));
  label->set_justify (GTK_JUSTIFY_RIGHT);
  frame->add (*label);
  vbox->pack_start (*frame, FALSE, FALSE);

  vbox = manage( new Gtk_VBox (FALSE, 5) );
  hbox->pack_start (*vbox, FALSE, FALSE);

  frame = manage( new Gtk_Frame ("Line wrapped label"));
  label = manage( new Gtk_Label ("This is an example of a line-wrapped label.  It " \
			 "should not be taking up the entire             " /* big space to test spacing */\
			 "width allocated to it, but automatically " \
			 "wraps the words to fit.  " \
			 "The time has come, for all good men, to come to " \
			 "the aid of their party.  " \
			 "The sixth sheik's six sheep's sick.\n" \
			 "     It supports multiple paragraphs correctly, " \
			 "and  correctly   adds "\
			 "many          extra  spaces. "));
  label->set_line_wrap (TRUE);
  frame->add (*label);
  vbox->pack_start (*frame, FALSE, FALSE);
  
  frame = manage( new Gtk_Frame("Filled, wrapped label") );
  label = manage( new Gtk_Label("This is an example of a line-wrapped, filled label.  " \
			 "It should be taking "\
			 "up the entire              width allocated to it.  " \
			 "Here is a seneance to prove "\
			 "my point.  Here is another sentence. "\
			 "Here comes the sun, do de do de do.\n"\
			 "    This is a new paragraph.\n"\
			 "    This is another newer, longer, better " \
			 "paragraph.  It is coming to an end, "\
			 "unfortunately.") );
  label->set_justify (GTK_JUSTIFY_FILL);
  label->set_line_wrap (TRUE);
  frame->add (*label);
  vbox->pack_start (*frame, FALSE, FALSE);
  
  frame = manage( new Gtk_Frame("Underlined label") );
  label = manage( new Gtk_Label("This label is underlined!\n"
			 "This one is underlined in quite a funky fashion") );
  label->set_justify (GTK_JUSTIFY_LEFT);
  label->set_pattern ("_________________________ _ _________ _ ______     __ _______ ___");
  frame->add (*label);
  vbox->pack_start (*frame, FALSE, FALSE);
  
  show_all ();
}

AppWindow::~AppWindow() {}

int main( int   argc,
          char *argv[] )
{
  /* Initialise GTK */
  Gtk_Main kit(&argc, &argv);
    
  /* Create a new window */
  AppWindow app;
    
  /* Enter the event loop */
  Gtk_Main::run ();
    
  return(0);
}
