/*
 *
 *  This documents the usage of Gtk_Menu_Helpers.
 *  peter
 *
 */

#include <gtk--/box.h>
#include <gtk--/main.h>
#include <gtk--/menu.h>
#include <gtk--/menubar.h>
#include <gtk--/window.h>

void void_void_cb(void)
{
  cout << "reached void_void_callback" << endl;
}

void void_int_cb(int num)
{
  cout << "reached void_int_callback, num = " << num << endl;
}

void edit_cb(string action)
{
  cout << "reached edit_callback, requested action " << action << endl;
}
void toggle_cb(string action)
{
  cout << "reached toggle_callback, requested action " << action << endl;
}
void destroy_cb(void)
{
  Gtk_Main::quit();
}


int main(int argc, char *argv[])
{
   Gtk_Main m(argc, argv);

   // menu subitems
   Gtk_AccelGroup *accgrp = Gtk_AccelGroup::create();

   Gtk_Menu    *fl_menu_file                = manage( new Gtk_Menu());
   Gtk_Menu    *fl_menu_edit                = manage( new Gtk_Menu());
   Gtk_Menu    *fl_menu_examples            = manage( new Gtk_Menu());
   Gtk_Menu    *fl_menu_help                = manage( new Gtk_Menu());
   Gtk_MenuBar *fl_menubar                  = manage( new Gtk_MenuBar());

   // Activate the helpers for easy menu setup.
   {
     using namespace Gtk_Menu_Helpers;

     MenuList fl_menulist_file     = fl_menu_file->items();
     MenuList fl_menulist_edit     = fl_menu_edit->items();
     MenuList fl_menulist_examples = fl_menu_examples->items();
     MenuList fl_menulist_help     = fl_menu_help->items();

     // Connecting non-default callback, getting the element's widget
     MenuElem filenew("_New", accgrp);
     fl_menulist_file.push_back(filenew);
     filenew.get_child()->activate.connect(bind(slot(void_int_cb), 123));

     // Usage of widget functions on menuitems through ->get_child()
     MenuElem fileopen("_Open", accgrp, slot(void_void_cb));
     fileopen.get_child()->set_state(GTK_STATE_INSENSITIVE);
     fl_menulist_file.push_back(fileopen);

     // Modifying a slot 
     fl_menulist_file.push_back(MenuElem("Save", accgrp, 
                                bind(slot(void_int_cb), 234)));
     fl_menulist_file.push_back(MenuElem("Save As", accgrp, 
                                slot(void_void_cb)));

     // How to install accelerators with '_' and accgrp
     fl_menulist_file.push_back(MenuElem("_Close", accgrp, 
                                slot(void_void_cb)));

     fl_menulist_file.push_back(SeparatorElem());
     fl_menulist_file.push_back(MenuElem("_Quit", accgrp, slot(destroy_cb)));


     fl_menulist_edit.push_back(MenuElem("Cut"));
     fl_menulist_edit.push_back(MenuElem("Copy"));
     fl_menulist_edit.push_back(MenuElem("Paste"));


     GSList *group = 0;
     RadioMenuElem fl_menuitem_example1(group, 
                                        "RadioItem Example 1", bind(slot(toggle_cb), string("radio example1")));
     RadioMenuElem fl_menuitem_example2(fl_menuitem_example1.group(), 
                                        "RadioItem Example 2", bind(slot(toggle_cb), string("radio example2")));
     RadioMenuElem fl_menuitem_example3(fl_menuitem_example1.group(), 
                                        "RadioItem Example 3", bind(slot(toggle_cb), (string)"radio example3"));
     CheckMenuElem fl_menuitem_example4("CheckItem Example", bind(slot(toggle_cb), (string)"check example"));
     fl_menuitem_example3.set_active(true);
     fl_menulist_examples.push_back(fl_menuitem_example1);
     fl_menulist_examples.push_back(fl_menuitem_example2);
     fl_menulist_examples.push_back(fl_menuitem_example3);
     fl_menulist_examples.push_back(SeparatorElem());
     fl_menulist_examples.push_back(fl_menuitem_example4);


     fl_menulist_help.push_back(MenuElem("About", accgrp, slot(void_void_cb)));


     // main-menuitems
     MenuElem fl_menuitem_file("_File", accgrp);
     MenuElem fl_menuitem_edit("Edit");
     MenuElem fl_menuitem_examples("Examples");
     MenuElem fl_menuitem_help("Help");

     // see usage of convenience functions on menuitems
     fl_menuitem_file.set_submenu(*fl_menu_file);
     fl_menuitem_edit.set_submenu(*fl_menu_edit);
     fl_menuitem_examples.set_submenu(*fl_menu_examples);
     fl_menuitem_help.set_submenu(*fl_menu_help);
     fl_menuitem_help.right_justify();

     // menubar
     fl_menubar->items().push_back(fl_menuitem_file);
     fl_menubar->items().push_back(fl_menuitem_edit);
     fl_menubar->items().push_back(fl_menuitem_examples);
     fl_menubar->items().push_back(fl_menuitem_help);
   }

   // vbox
   Gtk_VBox        *fl_vbox          = manage( new Gtk_VBox() );
   fl_vbox->pack_start (*fl_menubar,  false,  false, 0);

   Gtk_Window *fl_window        = manage( new Gtk_Window() );

   fl_window->destroy.connect(slot(destroy_cb));
   fl_window->set_title ("MenuDemo");
   fl_window->set_default_size (600, 400);
   fl_window->add (*fl_vbox);
   fl_window->add_accel_group (*accgrp);
   fl_window->set_policy (false, true, false);
   fl_window->show_all();

   m.run();
}
