#include <stdio.h>
#include <gtk--/text.h>
#include <gtk--/main.h>
#include <gtk--/window.h>
#include <gtk--/box.h>
#include <gtk--/button.h>
#include <gtk--/checkbutton.h>
#include <gtk--/buttonbox.h>
#include <gtk--/table.h>
#include <gtk--/scrollbar.h>
#include <gtk--/separator.h>

class AppWindow : public Gtk_Window {
  Gtk_Text text;
  Gtk_CheckButton edit_check;
  Gtk_CheckButton wrap_check;
public:
  AppWindow();
  ~AppWindow();

  void text_toggle_editable () {
    text.set_editable(edit_check.get_active());
  }

  void text_toggle_word_wrap () {
    text.set_word_wrap(wrap_check.get_active());
  }

};

void close_application( GtkWidget *widget, gpointer data )
{
       gtk_main_quit();
}


AppWindow::AppWindow() :
  Gtk_Window(GTK_WINDOW_TOPLEVEL), edit_check("Editable"),
  wrap_check("Wrap Words")
{
  Gtk_VBox *box1;
  Gtk_VBox *box2;
  Gtk_HButtonBox *hbox;
  Gtk_Button *button;
  Gtk_HSeparator *separator;
  Gtk_Table *table;
  Gtk_VScrollbar *vscrollbar;
  GdkColormap *cmap;
  GdkColor colour, black, white;
  GdkFont *fixed_font;

  FILE *infile;

 
  set_usize(600, 500);
  set_policy ( TRUE, TRUE, FALSE);  
  set_title ("Text Widget Example");
  set_border_width (0);
  
  
  box1 = manage( new Gtk_VBox (FALSE, 0) );
  add(*box1);
  
  box2 = manage( new Gtk_VBox (FALSE, 10) );
  box2->set_border_width (10);
  box1->pack_start (*box2, TRUE, TRUE, 0);
  
  table = manage( new Gtk_Table (2, 2, FALSE) );
  table->set_row_spacing (0, 2);
  table->set_col_spacing (0, 2);
  box2->pack_start (*table, TRUE, TRUE, 0);
  
  /* Create the GtkText widget */
  text.set_editable (TRUE);
  table->attach (text, 0, 1, 0, 1,
		    GTK_EXPAND | GTK_SHRINK | GTK_FILL,
		    GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);

  /* Add a vertical scrollbar to the GtkText widget */
  vscrollbar = manage( new Gtk_VScrollbar (*text.get_vadjustment()));
  table->attach ( *vscrollbar, 1, 2, 0, 1,
		    GTK_FILL, GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);

  /* Get the system colour map and allocate the colour red */
  cmap = gdk_colormap_get_system();
  colour.red = 0xffff;
  colour.green = 0;
  colour.blue = 0;
  if (!gdk_color_alloc(cmap, &colour)) {
    g_error("couldn't allocate colour");
  }
  white.red = 0xffff;
  white.green= 0xffff;
  white.blue= 0xffff;
  if (!gdk_color_alloc(cmap, &white)) {
    g_error("couldn't allocate colour");
  }
  black.red = 0;
  black.green= 0;
  black.blue= 0;
  if (!gdk_color_alloc(cmap, &black)) {
    g_error("couldn't allocate colour");
  }

  /* Load a fixed font */
  fixed_font = gdk_font_load ("-misc-fixed-medium-r-*-*-*-140-*-*-*-*-*-*");

  /* Realizing a widget creates a window for it, ready for us to insert some text */
  text.realize ();

  /* Freeze the text widget, ready for multiple updates */
  text.freeze ();
  
  /* Insert some coloured text */
  text.insert ( NULL, &black, &white,
		   "Supports ", -1);
  text.insert ( NULL, &colour, &white,
		   "colored ", -1);
  text.insert ( NULL, &black, &white,
		   "text and different ", -1);
  text.insert ( fixed_font, &black, &white,
		   "fonts\n\n", -1);
  
  /* Load the file text.c into the text window */

  infile = fopen("text.c", "r");
  
  if (infile) {
    char buffer[1024];
    int nchars;
    
    while (1)
      {
	nchars = fread(buffer, 1, 1024, infile);
	text.insert ( fixed_font, NULL,
			 NULL, buffer, nchars);
	
	if (nchars < 1024)
	  break;
      }
    
    fclose (infile);
  }

  /* Thaw the text widget, allowing the updates to become visible */  
  text.thaw ();
  
  hbox = manage( new Gtk_HButtonBox () );
  box2->pack_start ( *hbox, FALSE, FALSE, 0);

  hbox->pack_start ( edit_check, FALSE, FALSE, 0);
  edit_check.toggled.connect(slot(this, &AppWindow::text_toggle_editable));
  edit_check.set_active(TRUE);

  hbox->pack_start (wrap_check, FALSE, TRUE, 0);
  wrap_check.toggled.connect(slot(this, &AppWindow::text_toggle_word_wrap));
  wrap_check.set_active(FALSE);

  separator = manage( new Gtk_HSeparator () );
  box1->pack_start ( *separator, FALSE, TRUE, 0);

  box2 = manage( new Gtk_VBox (FALSE, 10) );
  box2->set_border_width (10);
  box1->pack_start ( *box2, FALSE, TRUE, 0);
  
  button = manage( new Gtk_Button ("close") );
  button->clicked.connect( Gtk_Main::quit.slot() );
  box2->pack_start ( *button, TRUE, TRUE, 0);
  button->set_flags( GTK_CAN_DEFAULT);
  button->grab_default ();

  show_all ();
}

AppWindow::~AppWindow() {}

int main (int argc, char *argv[])
{
  Gtk_Main m(argc, argv);
  AppWindow app;

  Gtk_Main::run();
  
  return(0);       
}
